"""
.. rubric:: Missing values
    The MissingType class implements a missing type which is used in the 
    internal representation. Types which allow missing values have to set the flag
    'has_missing' to True. 

    The only instance from MissingType that should be used is Missing (similar to None).
    This way one can test for equivalence by using 'variable_name is Missing'. 

    MissingType implements most type overloads to implement missing value behaviour. 
    E.g., Missing + 3 = Missing,   Missing & False = False, Missing | True = True,
    Missing == Missing = False, etc., similar to SQL NULL. 

    !!!To make it possible to have multiple Missing values in the same set, but use the same
    instance, __hash__ returns an incrementing value, and __eq__ returns False. 
    We have to make sure that this keeps working in Python (i.e. that set will not start to 
    use a pointer comparision shortcut). Maybe we can find a better solution in the future. 
"""
import util

hash_counter = 0
class MissingType(object):#{{{
    """MissingType value object, representing not-existing or unknown values.
       Singleton value is available in rtypes.Missing"""

    __slots__ = []

    def __new__(cls):
        if not hasattr(cls, "Missing"):
            cls.Missing = object.__new__(cls)

        return cls.Missing

    def __getnewargs__(self):
        return tuple()

    def __add__(self, other):
        return self

    def __radd__(self, other):
        return self

    def __sub__(self, other):
        return self

    def __rsub__(self, other):
        return self

    def __mul__(self, other):
        return self

    def __rmul__(self, other):
        return self

    def __mod__(self, other):
        return self

    def __rmod__(self, other):
        return self

    def __div__(self, other):
        return self

    def __rdiv__(self, other):
        return self

    def __and__(self, other):
        if(other is False or (isinstance(other,numpy.bool_) and other == False)):
            return False
        elif(isinstance(other,numpy.ndarray)):
            return other.__and__(self)
        return self

    def __rand__(self, other):
        if(other is False or (isinstance(other,numpy.bool_) and other == False)):
            return False
        elif(isinstance(other,numpy.ndarray)):
            return other.__rand__(self)
        return self

    def __or__(self, other):
        if(other is True or (isinstance(other,numpy.bool_) and other == True)):
            return True
        elif(isinstance(other,numpy.ndarray)):
            return other.__ror__(self)
        return self

    def __ror__(self, other):
        if(other is True or (isinstance(other,numpy.bool_) and other == True)):
            return True
        elif(isinstance(other,numpy.ndarray)):
            return other.__or__(self)
        return self

    def __xor__(self, other):
        return self

    def __rxor__(self, other):
        return self

    def __eq__(self, other):
        return self

    def __ne__(self, other):
        return self

    def __le__(self, other):
        return self

    def __ge__(self, other):
        return self

    def __lt__(self, other):
        return self

    def __gt__(self, other):
        return self

    def __invert__(self):
        return self

    def __pos__(self):
        return self

    def __neg__(self):
        return self

    def __abs__(self):
        return self

    def __nonzero__(self):
        return False
    
    def __hash__(self):
        global hash_counter
        hash_counter += 1
        return hash_counter

    def __repr__(self):
        return "--"

    def __call__(self, *seq, **kwds):
        return self

#Singleton value for _MissingType
Missing = MissingType()




from utils import util, context, infix
from parser_objs import *
import inspect
import representor
allowed_clss = (representor.Representor, context.Context, infix.Infix, int, long, float, str)


class CommandLineScanner(GenericScanner):#{{{
    def tokenize(self, input):
        self.rv = []
        GenericScanner.tokenize(self,input)
        return self.rv

    def t_anumber_0(self,s):
        r" \d*\.\d+([eE][\+\-]\d+)?[jJ]? | \d+\.([eE][\+\-]\d+)?[jJ]? | \d+[eE][\+\-]\d+[jJ]? "
        t = Token(type="float",attr=s)
        self.rv.append(t)
   
    def t_anumber_1(self,s):
        """ 
            0[bB][01]+[lL]? | 0[xX][0-9a-fA-F]+[lL]? | 0[oO][0-7]*[lL]? | \d+[lL]? 
        """
        t = Token(type="integer",attr=s)
        self.rv.append(t)

    def t_bkeywords(self, s):
        r' and[^a-zA-Z\d]+ | as[^a-zA-Z\d]+ | assert[^a-zA-Z\d]+  | break[^a-zA-Z\d]+  | class[^a-zA-Z\d]+  | continue[^a-zA-Z\d]+  | def[^a-zA-Z\d]+  | del[^a-zA-Z\d]+  | elif[^a-zA-Z\d]+  | else[^a-zA-Z\d]+  | except[^a-zA-Z\d]+  | exec[^a-zA-Z\d]+  | finally[^a-zA-Z\d]+  | for[^a-zA-Z\d]+  | from[^a-zA-Z\d]+  | global[^a-zA-Z\d]+  | if[^a-zA-Z\d]+  | import[^a-zA-Z\d]+  | in[^a-zA-Z\d]+  | is[^a-zA-Z\d]+  | lambda[^a-zA-Z\d]+  | not[^a-zA-Z\d]+  | or[^a-zA-Z\d]+  | pass[^a-zA-Z\d]+  | print[^a-zA-Z\d]+  | raise[^a-zA-Z\d]+  | return[^a-zA-Z\d]+  | try[^a-zA-Z\d]+  | while[^a-zA-Z\d]+  | with[^a-zA-Z\d]+  | yield[^a-zA-Z\d]+  '
        t = Token(type=s)
        self.rv.append(t)

    def t_cidentifier(self,s):
        r' [a-zA-Z_][a-zA-Z_\d]* '
        t = Token(type='identifier',attr=s)
        self.rv.append(t)

    def t_dsymbol_0(self,s):
        r' \"\"\" | \+\= | \-\= | \*\= | \/\= | \/\/\= | \%\= | \&\= | \|\= | \^\= | \>\>\= | \<\<\= | \*\*\= '
        t = Token(type=s)
        self.rv.append(t)

    def t_esymbol_1(self, s):
        r' \+ | \- | \*\* | \* | \/\/ | \/ | \% | \<\< | \>\> | \& | \| | \^ | \~ | \< | \> | \<\= | \>\= | \=\= | \!\= | \<\> '
        t = Token(type=s)
        self.rv.append(t)

    def t_fsymbol_2(self,s):
        r' \( | \) | \[ | \] | \{ | \} | \@ | \, | \: | \. | \` | \= | \; | \" | \''
        t = Token(type=s)
        self.rv.append(t)#}}}

    def t_gwhitespace(self,s):
        r' [\t\s\n]+ '
        t = Token(type="whitespace",attr=s)
        self.rv.append(t)
        pass

def merge_tokens(tokens):
    res = []
    for token in tokens:
        if token.attr is None:
            res.append(str(token.type))
        else:
            res.append(str(token.attr))
    return "".join(res)

def process_tokens(tokens):
    if not tokens:
        return tokens
    pos = 0
    while pos < len(tokens):
        token = tokens[pos]
        if pos < len(tokens) - 1:
            nexttoken = tokens[pos + 1]
        else:
            nexttoken = None
       
        if token == '"' or token.type == '"""' or token.type == "'":
            try:
                endpos = tokens.index(token.type,pos+1)
                ntoken = Token(type="string", attr=merge_tokens(tokens[(pos + 1):endpos]))
                tokens = tokens[:pos] + [ntoken] + tokens[(endpos + 1):]
            except ValueError:
                tokens[pos] = Token(type="incomplete_string", attr=merge_tokens(tokens[(pos + 1):]))
                tokens = tokens[:(pos + 1)]
        elif token == "whitespace":
            del tokens[pos]
            pos -= 1
        pos = pos + 1
    return tokens

class CommandLineParser(GenericParser):#{{{
    def __init__(self, start="simple_stmt"):
        GenericParser.__init__(self, start)

    def p_atom_0(self,args):
        """ 
            atom ::= identifier
            atom ::= literal
            atom ::= enclosure
        """
        return AST(type="atom",kids=args[:1])
        
    def p_enclosure_0(self,args):
        """
            enclosure ::= parenth_form
            enclosure ::= list_display
            enclosure ::= generator_expression
            enclosure ::= dict_display
            enclosure ::= set_display
            enclosure ::= string_conversion
            enclosure ::= yield_atom
        """
        return AST(type="enclosure",kids=args[:1])

    def p_literal_0(self,args):
        """
           literal ::= integer
           literal ::= string
           literal ::= incomplete_string
           literal ::= float
        """
        return AST(type="literal",kids=args[:1])

    def p_parenth_form_0(self,args):
        """
            parenth_form ::= ( expression_list )
            parenth_form ::= ( )
        """
        return AST(type="parenth_form",kids=args[1:-1])


    def p_list_display_0(self,args):
        """
            list_display ::= [ ]
            list_display ::= [ expression_list ]
            list_display ::= [ list_comprehension ] 
        """
        return AST(type="list_display",kids=args[1:-1])
   
    def p_list_comprehension(self, args):
        r" list_comprehension ::= expression list_for "
        return AST(type="list_comprehension",kids=args)

    def p_list_for(self, args):
        """
            list_for ::= for target_list in old_expression_list
            list_for ::= for target_list in old_expression_list list_iter
        """
        return AST(type="list_for",kids=args[1:2] + args[3:])
           

    def p_old_expression_list(self,args):
        """
            old_expression_list ::= old_expression 
            old_expression_list ::= old_expression_list , old_expression
        """
        return AST(type="old_expression_list",kids=args[:1] + args[2:])

    def p_old_expression(self,args):
        """
            old_expression ::= or_test
            old_expression ::= old_lambda_form
        """
        return AST(type="old_expression",kids=args[:1])


    def p_list_iter(self,args):
        """
            list_iter ::= list_for
            list_iter ::= list_if
        """
        return AST(type="list_iter",kids=args[:1])

    def p_list_if(self,args):
        """
            list_if ::= if old_expression
            list_if ::= if old_expression list_iter
        """
        return AST(type="list_if",kids=args[1:])

    def p_comprehension(self,args):
        r" comprehension ::= expression comp_for "
        return AST(type="comprehension",kids=args)

    def p_comp_for(self,args):
        """
            comp_for ::= for target_list in or_test
            comp_for ::= for target_list in or_test comp_iter
        """
        return AST(type="comp_for",kids=args[1:2] + args[3:])

    def p_comp_iter(self, args):
        """
            comp_iter ::= comp_for
            comp_iter ::= comp_if
        """
        return AST(type="comp_iter",kids=args[:1])

    def p_comp_if(self,args):
        """
            comp_if ::= if expression_nocond
            comp_if ::= if expression_nocond comp_iter
        """ 
        return AST(type="comp_if",kids=args[1:])


    def p_generator_expression(self,args):
        r" generator_expression ::= ( expression comp_for ) "
        return AST(type="generator_expression",kids=args[1:3])

    def p_string_conversion_0(self,args):
        """
            string_conversion ::= ` expression_list `
        """
        return AST(type="string_conversion",kids=args[1:-1])

    def p_primary(self,args):
        """
            primary ::= atom
            primary ::= attributeref
            primary ::= subscription
            primary ::= slicing
            primary ::= partial_slicing
            primary ::= call
        """
        return AST(type="primary",kids=args[:1])

    def p_attributeref(self,args):
        """
            primary ::= primary . identifier
            primary ::= primary .
        """
        return AST(type="attributeref",kids=args[:1] + args[2:])

    def p_subscription(self,args):
        """
            subscription ::= primary [ expression_list ] 
        """
        return AST(type="subscription",kids=args[:1] + args[2:3])


    def p_partial_slicing(self,args):
        """
             partial_slicing ::= primary [ slice_list
        """
        return AST(type="partial_slicing",kids=args[:1] + args[2:3])

    def p_slicing(self, args):
        """
            slicing ::= simple_slicing
            slicing ::= extended_slicing
        """
        return AST(type="slicing",kids=args[:1])

    def p_simple_slicing(self,args):
        """
            simple_slicing ::= primary [ short_slice ] 
        """
        return AST(type="simple_slicing",kids=args[:1] + args[2:3])

    def p_extended_slicing(self, args):
        """
            extended_slicing ::= primary [ slice_list ]
        """
        return AST(type="extended_slicing",kids=args[:1] + args[2:3])

    def p_slice_list(self, args):
        """
            slice_list ::= slice_item
            slice_list ::= slice_list , slice_item
        """
        return AST(type="slice_list",kids=args[:1] + args[2:])

    def p_slice_item(self, args):
        """
            slice_item ::= expression
            slice_item ::= proper_slice
            slice_item ::= ellipsis
        """
        return AST(type="slice_item",kids=args[:1])

    def p_proper_slice(self, args):
        """
            proper_slice ::= short_slice
            proper_slice ::= long_slice
        """
        return AST(type="proper_slice",kids=args[:1])

    def p_short_slice(self, args):
        """
            short_slice ::= :
            short_slice ::= expression :
            short_slice ::= : expression
            short_slice ::= expression : expression
        """
        none = Token(type="object",attr=None)
        if(len(args) == 1):
            kids = [none, none]
        elif(args[0] == "expression"):
            if len(args) > 2:
                kids = [args[0], args[2]]
            else:
                kids = [args[0], none]
        else:
            kids = [none, args[1]]
        return AST(type="short_slice",kids=kids)
        

    def p_long_slice(self, args):
        """
            long_slice ::= short_slice :
            long_slice ::= short_slice : expression
        """
        return AST(type="extended_slicing",kids=args[:1] + args[2:])

    def p_ellipsis(self, args):
        """
            ellipsis ::= . . .
        """
        return Token(type ="elllipsis")

    

    def p_call(self,args):
        """
            call ::= primary ( argument_list )
        """
        #FIXME
        return AST(type="call",kids=args[:1] + args[2:3])


    def p_argument_list(self, args):
        """
            argument_list ::= positional_arguments
            argument_list ::= positional_arguments , keyword_arguments
            argument_list ::= keyword_arguments
        """
        #FIXME
        return AST(type="argument_list",kids=args[:1] + args[2:])

    def p_positional_arguments(self, args):
        """
            positional_arguments ::= expression
            positional_arguments ::= positional_arguments , expression
        """
        return AST(type="positional_arguments",kids=args[:1] + args[2:])

    def p_keyword_arguments(self, args):
        """
            keyword_arguments ::= keyword_item
            keyword_arguments ::= keyword_arguments , keyword_item
        """
        return AST(type="keyword_arguments",kids=args[:1] + args[2:])

    def p_keyword_item(self, args):
        r" keyword_item ::= identifier = expression "
        return AST(type="keyword_item",kids=args[:1] + args[2:])

    def p_conditional_expression(self, args):
        """
            conditional_expression ::= or_test if or_test else expression
        """
        if(len(args) > 1):
            return AST(type="conditional_expression",kids=args[:1] + args[3:4] + args[5:])
        else:
            return AST(type="conditional_expression",kids=args[:1])


    def p_expression(self,args):
        """
            expression ::= conditional_expression
            expression ::= lambda_form
        """
        return AST(type="expression",kids=args[:1])


    def p_power(self,args):
        """
            power ::= primary
            power ::= primary ** u_expr
        """
        return AST(type="power",kids=args[:1] + args[2:])

    def p_u_expr(self, args):
        """
            u_expr ::= power
            u_expr ::= - u_expr
            u_expr ::= + u_expr
            u_expr ::= ~ u_expr
        """
        return AST(type="u_expr",kids=args)

    def p_m_expr(self, args):
        """
            m_expr ::= u_expr
            m_expr ::= m_expr * u_expr
            m_expr ::= m_expr // u_expr
            m_expr ::= m_expr / u_expr
            m_expr ::= m_expr % u_expr
        """
        return AST(type="m_expr",kids=args)

    def p_a_expr(self, args):
        """
            a_expr ::= m_expr
            a_expr ::= a_expr + m_expr
            a_expr ::= a_expr - m_expr
        """
        return AST(type="a_expr",kids=args)

    def p_shift_expr(self, args):
        """
            shift_expr ::= a_expr
            shift_expr ::= shift_expr << a_expr
            shift_expr ::= shift_expr >> a_expr
        """
        return AST(type="shift_expr",kids=args)


    def p_and_expr(self, args):
        """
            and_expr ::= shift_expr
            and_expr ::= and_expr & shift_expr
        """
        return AST(type="and_expr",kids=args)

    def p_xor_expr(self, args):
        """
            xor_expr ::= and_expr
            xor_expr ::= xor_expr & and_expr
        """
        return AST(type="xor_expr",kids=args)

    def p_or_expr(self, args):
        """
            or_expr ::= xor_expr
            or_expr ::= or_expr | xor_expr
        """
        return AST(type="or_expr",kids=args)


    def p_comparision(self, args):
        """
            comparison ::= or_expr
            comparison ::= comparison < or_expr
            comparison ::= comparison > or_expr
            comparison ::= comparison == or_expr
            comparison ::= comparison >= or_expr
            comparison ::= comparison <= or_expr
            comparison ::= comparison <> or_expr
            comparison ::= comparison != or_expr
            comparison ::= comparison is or_expr
            comparison ::= comparison is not or_expr
            comparison ::= comparison in or_expr
            comparison ::= comparison not in or_expr
        """
        return AST(type="comparison",kids=args)

    def p_not_test(self, args):
        """
            not_test ::= comparison
            not_test ::= not not_test
        """
        return AST(type="not_test",kids=args[:1] + args[2:])

    def p_and_test(self, args):
        """
            and_test ::= not_test
            and_test ::= and_test and not_test
        """
        return AST(type="and_test",kids=args[:1] + args[2:])

    def p_or_test(self, args):
        """
            or_test ::= and_test
            or_test ::= or_test or and_test
        """
        return AST(type="or_test",kids=args[:1] + args[2:])

    def p_conditional_expression(self, args):
        """
            conditional_expression ::= or_test 
            conditional_expression ::= or_test if or_test else expression
        """
        return AST(type="conditional_expression",kids=args[:1] + args[2:3] + args[4:])

    def p_expression(self, args):
        """
            expression ::= conditional_expression
            expression ::= lambda_form
        """
        return AST(type="expression",kids=args[:1])

    def p_lambda_form(self,args):
        """
            lambda_form ::= lambda : expression 
            lambda_form ::= lambda parameter_list : expression
        """
        if len (args) == 3:
            return AST(type="lambda_form",kids=args[2:])
        else:
            return AST(type="lambda_form",kids=args[1:2] + args[3:])

    def p_old_lambda_form(self,args):
        """
            old_lambda_form ::= lambda : old_expression 
            old_lambda_form ::= lambda parameter_list : old_expression
        """
        if len (args) == 3:
            return AST(type="old_lambda_form",kids=args[2:])
        else:
            return AST(type="old_lambda_form",kids=args[1:2] + args[3:])


    def p_expression_list(self, args):
        """
            expression_list ::= expression
            expression_list ::= expression_list , expression
        """
        return AST(type="expression_list",kids=args[:1] + args[2:])

    def p_simple_stmt(self, args):
        """
            simple_stmt ::= expression_stmt
            simple_stmt ::= assert_stmt
            simple_stmt ::= assignment_stmt
            simple_stmt ::= augmented_assignment_stmt
            simple_stmt ::= pass_stmt
            simple_stmt ::= del_stmt
            simple_stmt ::= print_stmt
            simple_stmt ::= return_stmt
            simple_stmt ::= yield_stmt
            simple_stmt ::= raise_stmt
            simple_stmt ::= break_stmt
            simple_stmt ::= continue_stmt
            simple_stmt ::= import_stmt
            simple_stmt ::= global_stmt
            simple_stmt ::= exec_stmt
        """
        return AST(type="simple_stmt",kids=args[:1])

    def p_expression_stmt(self, args):
        """
            expression_stmt ::= expression_list
        """
        return AST(type="expression_stmt", kids=args[:1])
    def p_assignemnt_stmt(self, args):
        """
            assignment_stmt ::= target_list = expression_list
            assignment_stmt ::= target_list = yield_expression
        """
        #FIXME
        return AST(type="assignment_stmt", kids=args[:1] + args[2:])
    
    def p_target_list(self, args):
        """
            target_list ::= target
            target_list ::= target_list , target
        """
        return AST(type="target_list", kids=args[:1] + args[2:])

    def p_target(self, args):
        """
            target ::= identifier
            target ::= ( target_list )
            target ::= [ target_list ] 
            target ::= attributeref
            target ::= subscription
            target ::= slicing
        """
        if(len(args) == 1):
            kids = args
        else:
            kids = args[1:2]
        return AST(type="target", kids=kids)
        

#}}}

class CommandLineASTRewriterPass1(GenericASTRewriter):#{{{
    def process(self, tree):
        ntree = self.postorder(tree)
        return ntree
    
    def n_slice_list(self, node):
        if(node.kids[0] == "slice_list"):
            node.kids = node.kids[0].kids + node.kids[1:]
        return node

    def n_positional_arguments(self, node):
        if(node.kids[0] == "positional_arguments"):
            node.kids = node.kids[0].kids + node.kids[1:]
        return node
    
    def n_keyword_arguments(self, node):
        if(node.kids[0] == "keyword_arguments"):
            node.kids = node.kids[0].kids + node.kids[1:]
        return node
   
    def n_target_list(self, node):
        if(node.kids[0] == "target_list"):
            node.kids = node.kids[0].kids + node.kids[1:]
        return node
    
    def n_expression_list(self, node):
        if(node.kids[0] == "expression_list"):
            node.kids = node.kids[0].kids + node.kids[1:]
        return node

class CommandLineASTRewriterPass2(GenericASTRewriter):#{{{
    def process(self, tree, ipshell):
        self.ipshell = ipshell
        ntree = self.postorder(tree)
        return ntree

    def objectify(self, node, pos, context=None):
        if node.kids[pos] == "identifier":
            if context is not None and node.kids[pos].attr == "_":
                node.kids[pos] = Token(type="object", attr= context)
            else:
                node.kids[pos] = Token(type="object", attr= get_from_namespace(node.kids[pos].attr, self.ipshell))
        return node

    def n_attributeref(self, node, context=None):
        node = self.objectify(node,0, context)
        if node.kids[0] != "object":
            return node
       
        obj = node.kids[0].attr
        if(len(node.kids) == 1):
            node.kids.append(Token(type="identifier",attr=""))
        elif node.kids[1] == "identifier" and hasattr(obj, node.kids[1].attr):
            res = getattr(obj, node.kids[1].attr)
            return Token(type="object",attr=res)

        return node

    def n_slicing(self, node, context=None):
        node.kids = node.kids[0].kids
        node = self.objectify(node,0, context)

        if node.kids[0] != "object" or not isinstance(node.kids[0].attr, allowed_clss):
            return node
        obj = node.kids[0].attr

        if node.kids[1] == "object":
            return Token(type="object",attr=obj[node.kids[1].attr])

        return node
      
    def x_partial_slicing(self, node, context=None):
        node.kids[0] = self.postorder(node.kids[0], context)
        node = self.objectify(node,0,context)

        if node.kids[0] == "object":
            context = node.kids[0].attr

        return self.postorder(node.kids[1],context)



    def n_float(self, node, context=None):
        return Token(type="object", attr=float(node.attr))

    def n_integer(self, node, context=None):
        if node.attr[-1] == 'l' or node.attr[-1] == 'L':
            return Token(type="object", attr=long(node.attr))
        else:
            return Token(type="object", attr=int(node.attr))

    def n_string(self, node, context=None):
        return Token(type="object", attr=node.attr)

    def n_short_slice(self, node, context):
        if all([kid == "object" for kid in node.kids]):
            return Token(type="object", attr=slice(*[kid.attr for kid in node.kids]))
        return node

    def n_slice_list(self, node, context=None):
        if(len(node.kids) == 1):
            return node.kids[0]
        if not all([kid == "object" for kid in node.kids]):
            return node

        return Token(type="object", attr= tuple([kid.attr for kid in node.kids]))

    def n_target_list(self, node, context=None):
        if(len(node.kids) == 1):
            return node.kids[0]
        return node

    def n_argument_list(self, node, context=None):
        nkids = []
        params = []
        keywords = {}
        for kid in node.kids:
            if kid == 'positional_arguments':
                if(all([skid == "object" for skid in kid.kids])):
                    params.extend([skid.attr for skid in kid.kids])
                else:
                    return node
            elif kid == 'keyword_arguments':
                if(not all([skid == "keyword_item" for skid in kid.kids])):
                    return node
                if(not all([skid.kids[1] == "object" for skid in kid.kids])):
                    return node
                for skid in kid.kids:
                    keywords[skid.kids[0].attr] = skid.kids[1].attr
        node.keywords = keywords
        node.params = params
        return node

    def n_assignment_stmt(self, node, context=None):
        #no assignments handled yet
        return node.kids[1]

    def n_call(self, node, context=None):
        if not (node.kids[0] == "object" and node.kids[1] == "argument_list" and hasattr(node.kids[1],'params')):
            return node

        method = node.kids[0].attr
        if(inspect.ismethod(method)):
           if not isinstance(method.im_self, allowed_clss):
               return node
           if isinstance(method.im_self, representor.Representor) and method.im_func.func_name in ['__str__','__nonzero__','Copy','__reduce__']:
               return node
           res = method(*node.kids[1].params, **node.kids[1].keywords)
           return Token(type="object", attr= res)


    def binoperator(self, node, context=None):
        if(len(node.kids) == 1):
            return node.kids[0]
        node = self.objectify(node,0, context)
        if node.kids[0] =="object" and node.kids[2] == "object":
            obj1 = node.kids[0].attr
            obj2 = node.kids[2].attr
            if(isinstance(obj1, allowed_clss) and isinstance(obj2, allowed_clss)):
                res = eval('obj1 ' + node.kids[1].type + ' obj2')
                return Token(type="object", attr= res)
        return node
    n_comparison = binoperator
    n_or_expr = binoperator
    n_and_expr = binoperator
    n_xor_expr = binoperator
    n_shift_expr = binoperator
    n_a_expr = binoperator
    n_m_expr = binoperator
    n_power = binoperator



    def n_removenode(self, node, context=None):
        if len(node.kids) == 1:
            return node.kids[0]
        return node
   
    n_expression_list = n_removenode
    n_expression = n_removenode
    n_conditional_expression = n_removenode
    n_and_test = n_removenode
    n_or_test = n_removenode
    n_not_test = n_removenode
    n_u_expr = n_removenode
    n_literal = n_removenode
    n_enclosure = n_removenode
    n_atom = n_removenode
    n_primary = n_removenode
    
    n_proper_slice = n_removenode
    n_slice_item = n_removenode
    n_simple_stmt = n_removenode
    n_expression_stmt = n_removenode
#}}}

class CommandLineASTInterpreterPass(GenericASTRewriter):#{{{
    def process(self, tree):
        ntree = self.toponly(tree)
        return ntree

    def toponly(self, node=None):
        name = 'n_' + self.typestring(node)
        if hasattr(self, name):
            func = getattr(self, name)
            node = func(node)
        else:
            node = self.default(node)
        return node

    def n_attributeref(self, node):
        if node.kids[0] == "object" and node.kids[1] == "identifier":
            return (node.kids[0].attr, node.kids[1].attr) 
        return node

    

#}}}


def get_from_namespace(attr, ipshell):
    if attr in ipshell.IP.user_ns:
        return ipshell.IP.user_ns[attr]
    else:
        return ipshell.IP.user_global_ns[attr]

def parseCommandLine(line, ipshell, cursor_pos=None, debug=False):
    if cursor_pos is None:
        parse_line = line
    else:
        parse_line = line[:cursor_pos]
    scanner = CommandLineScanner()
    tokens = scanner.tokenize(parse_line)
    if(debug):
        print 1, tokens
    tokens = process_tokens(tokens)
    if(debug):
        print 2, tokens
    parser = CommandLineParser()
    tree = parser.parse(tokens)
    if(debug):
        print 3, tree
    
    rewriter = CommandLineASTRewriterPass1()
    tree = rewriter.process(tree)
    if(debug):
        print 4, tree
   
    rewriter = CommandLineASTRewriterPass2()
    tree = rewriter.process(tree,ipshell)
    if(debug):
        print 5, tree

    interpreter = CommandLineASTInterpreterPass()
    res = interpreter.process(tree)
    if(debug):
        print 6, res
    return res



import os
import csv

import wrapper
_delay_import_(globals(),"..itypes","rtypes")
_delay_import_(globals(),"..slices")
_delay_import_(globals(),"wrapper_py","Result","rep")
_delay_improt_(globals(),"wrapper_tsv","read_tsv")
_delay_import_(globals(),"..utils","cutils","util")

def read_hughes(path):
    experiments = read_tsv(path + "/experiment_list.txt", skiprows=2)
    return (experiments, HughesRepresentor(path + "/data_expts1-300_geneerr.txt").A, 
                         HughesRepresentor(path + "/control_expts1-63_geneerr.txt").A)

    

htype = "tuple(gene_official=[genes:*](string[]), gene_symbols=[genes:*](string[]), " + \
       "knockouts=[knockouts:*](string[]), pval=[genes:*]([knockouts:*](real64$)), " + \
               "ratio=[genes:*]([knockouts:*](real64$)), intensity=[genes:*]([knockouts:*](real64$)), " + \
               "ratioerr=[genes:*]([knockouts:*](real64$)))"

class HughesRepresentor(wrapper.SourceRepresentor):
    _select_indicator = None
    _select_executor = None
    
    def __init__(self, filename):
        self._filename = filename
        
        ntype = rtypes.createType(htype)
        tslices = (slices.Slice("data", ntype),)

        all_slices = dict([(slice.id, slice) for slice in tslices])
        wrapper.SourceRepresentor.__init__(self, all_slices, tslices)

    def pyexec(self, executor):
        file = open(self._filename)

        #skip first two lines
        file.readline()
        file.readline()


        reader = csv.reader(file, delimiter='\t')
        knockout_names = reader.next()[4::4]
        meas_names = reader.next()[2:]
        data = [row for row in reader]

        data = util.darray(data,object,2)
        gene_names = data[:, 0]
        gene_symbols = data[:,1]
        intensity = data[:, 2:-1:4]
        ratio = data[:, 3::4]
        ratio_err = data[:, 4::4]
        pval = data[:, 5::4]

        results = (gene_names, gene_symbols, knockout_names, pval, ratio, intensity, ratio_err)
        res = {}
        res[self._active_slices[0].id] = results
        return Result(res)





import unittest
import numpy

from ibidas.utils import util
from ibidas import *


class TestTutorial(unittest.TestCase):
    
    def test_yeastract_inoutdegree(self):
        url = "http://www.yeastract.com/download/RegulationTwoColumnTable_Documented_20111009.tsv.gz"
        yeastract = Read(Fetch(url))
        
        str(yeastract)
        yeastract = yeastract/('trans_factor','target')
        yeastract = yeastract%'tftargets'
        str(yeastract)
        yeastract = yeastract.Rename('trans_factor','target')
        yeastract = yeastract.DimRename('tftargets')
        str(yeastract)
        yeastract = yeastract.Copy()
        
        gyeastract = yeastract.GroupBy(_.trans_factor)
        gyeastract = yeastract.GroupBy(yeastract.trans_factor)
        str(gyeastract)
        
        gyeastract = gyeastract.Get(_.trans_factor, _.target.Count()/"out_degree")
        str(gyeastract)

        yeastract = Get.yeast.yeastract()
        yeastract.GroupBy(_.trans_factor).target.Count()()
        yeastract.GroupBy(_.target).trans_factor.Count()()



    def test_yeastract_inoutdegreeb(self):
        def yeastract2(url="http://www.yeastract.com/download/RegulationTwoColumnTable_Documented_20111009.tsv.gz"):
            """Downloads documented transcription factor regulation interactions from yeastract"""

            res = Read(Fetch(url),dtype="[tftargets:*]<(trans_factor=bytes, target=bytes)")
            return res.Copy()
        Get.register(yeastract2)



    def test_tutorial_chromodist(self):
        return
        rtype = """[feats:*]<(sgdid=bytes, feat_type=bytes, feat_qual=bytes, feat_name=bytes, gene_name=bytes,
                gene_aliases=bytes, feat_parent_name=bytes, sgdid_alias=bytes, chromosome=bytes,
                start=bytes, stop=bytes, strand=bytes[1], genetic_pos=bytes, coordinate_version=bytes[10],
                sequence_version=bytes, description=bytes)"""

        res = Read(Fetch("http://downloads.yeastgenome.org/curation/chromosomal_feature/SGD_features.tab"),dtype=rtype)

        resx = res/{'f3': 'feat_name', 'f8':'chromosome', 'f9':'start'}
        str(resx)

        res = res.To(_.start, _.stop, Do=_.Cast("int?"))
        res = res.To(_.genetic_pos,   Do=_.Cast("real64?"))
        
        resy = res.To(_.gene_aliases,  Do=_.Each(_.split('|')).Elems()[_ != ""])


        splitfunc = lambda x: x.split('|')
        resx = res.To(_.gene_aliases,  Do=_.Each(splitfunc, dtype="[aliases:~]<bytes").Elem()[_ != ""])
        
        yeast_feats = resy.Copy()
        yeast_feats = Get.yeast.genomic_feats()

        yeastract = Get.yeast.yeastract()
        yeast_feats = yeast_feats.To(_.feat_name, Do=_.Each(str.upper))
        yeastract = yeastract.To(_.target,         Do=_.Each(str.upper))

        tf_feat = yeastract.Match(yeast_feats, _.target, _.feat_name)
        str(tf_feat)
        util.debug_here()

        self.assertTrue((yeastract.target.Count() - tf_feat.target.Count()) == 74)

        nonmatched = (yeastract.target.Set() - tf_feat.target.Set()).Elem()
        str(nonmatched)


        str(nonmatched.In(yeast_feats.gene_name.Each(str.upper)))
        str(nonmatched |In| yeast_feats.gene_name.Each(str.upper))
        str(nonmatched |In| yeast_feats.gene_aliases.Each(str.upper))
        str(Any(nonmatched |In| yeast_feats.gene_aliases.Each(str.upper)))
        str(nonmatched |In| yeast_feats.gene_aliases.Flat().Each(str.upper))
       
        nonmatched_feats = nonmatched.Match(yeast_feats.Flat(), _.target, _.gene_aliases.Each(str.upper))
        str(nonmatched_feats)


        str(tf_feat.Unique(_.trans_factor, _.target).target.Count())

        nonmatched_unique = nonmatched_feats.GroupBy(_.target)[Count(_.feat_name) == 1].target
        str(nonmatched_unique)

        yt_nm = yeastract[_.target |In| nonmatched_unique]

        yeast_feats = yeast_feats.Get("*", _.gene_aliases/"gene_aliases2")
        yeast_feats = yeast_feats.To(_.gene_aliases, Do=_.Array())
        res = yt_nm.Match(yeast_feats.Flat(), _.target, _.gene_aliases2).Without(_.gene_aliases2)
        
        str(res)
        util.debug_here()
        tf_feat = Stack(tf_feat, res).Copy()
        Save(tf_feat, 'tf_feat.dat')
        tf_feat = Load('tf_feat.dat')


        tf_feat = tf_feat.GroupBy(_.trans_factor, _.chromosome)
        res = tf_feat.Get(_.trans_factor, _.chromosome, _.sgdid.Count()/"count", _.start).Copy()
        str(res)

        str(Corr(res.count))
        str(Corr(res.count.Transpose()))

        res = res.To(_.chromosome, Do=_.Cast("int?"))

        str(res.Sort(_.chromosome).Get(_.chromosome, Corr(_.count.Transpose()/"chromo_corr")))
        str(res.Get(_.chromosome, _.count.Sum("gtrans_factor")))
        str(res.count.Sort().Sum("gtrans_factor"))
        normalized_counts = res.count.Cast("real64") / res.count.Sum("gtrans_factor")
        str(normalized_counts.Sort().Sum("gtrans_factor"))


        normalized_counts.Sort().Sum("gtrans_factor")()




    def test_tutorial_chromodist2(self):
        yeastract = Get.yeast.yeastract()        
        rtype = """[feats:*]<(sgdid=bytes, feat_type=bytes, feat_qual=bytes, feat_name=bytes, gene_name=bytes,
                    gene_aliases=bytes, feat_parent_name=bytes, sgdid_alias=bytes, chromosome=bytes,
                    start=bytes, stop=bytes, strand=bytes[1], genetic_pos=bytes, coordinate_version=bytes[10],
                    sequence_version=bytes, description=bytes)"""
        
        
        res = Read(Fetch("http://downloads.yeastgenome.org/curation/chromosomal_feature/SGD_features.tab"),dtype=rtype)

        res = res/{'f3': 'feat_name', 'f8':'chromosome', 'f9':'start'}


        res = res.To(_.start, _.stop, Do=_.Cast("int?"))
        res = res.To(_.genetic_pos,   Do=_.Cast("real64?"))

        splitfunc = _.split('|')
        res.gene_aliases.Each(splitfunc).Elems()[_ != ""]

        
        dtype = "[aliases:~]<bytes"
        splitfilter = _.Each(splitfunc, dtype=dtype).Elems()[_ != ""]
        yeast_feats = res.To(_.gene_aliases, Do=splitfilter).Copy()


        tf_feat = yeastract |Match(_.target.Each(str.upper), _.feat_name.Each(str.upper))| yeast_feats
        str(tf_feat)

        str(yeastract.target.Count() - tf_feat.target.Count())

        d1 = Rep([1,2,3,3])
        d2 = Rep([1,3,3])
        d1 |Match| d2

        str(yeast_feats.feat_name[_ != ""].Get(_.Count() == _.Unique().Count()))
        str((yeastract |Except| tf_feat.Get(_.trans_factor, _.target)).Count())
        str((yeastract |Except| tf_feat.Get(*yeastract.Names)).Count())

        nonmatched = yeastract.target |Except| tf_feat.target
        str(nonmatched)

        non_matched = (yeastract.target.Set() - tf_feat.target.Set()).Elem()
        nonmatched = nonmatched.Each(str.upper)

        str(nonmatched |In| yeast_feats.gene_name.Each(str.upper))

        str(nonmatched.Each(str.upper) |In| yeast_feats.gene_aliases.Each(str.upper))

        str(Any(nonmatched |In| yeast_feats.gene_aliases.Each(str.upper)))

        nonmatched_feats = nonmatched |Match(_.target, _.gene_aliases.Each(str.upper))| yeast_feats.Flat()
        str(nonmatched_feats)


        unique_gene_aliases = yeast_feats.Flat().GroupBy(_.gene_aliases)[Count(_.feat_name) == 1].gene_aliases
        name_alias_list = yeast_feats[_.gene_aliases |In| unique_gene_aliases]
        convert_table = name_alias_list.Get(_.gene_aliases.Each(str.upper), _.feat_name).Flat()
        yeastract = yeastract.To(_.target, Do=_.Each(str.upper).TakeFrom(convert_table, keep_missing=True))
        tf_feat = yeastract |Match(_.target.Each(str.upper), _.feat_name.Each(str.upper))| yeast_feats

        str((yeastract |Except| tf_feat.Get(*yeastract.Names)).Count())


        name_alias_list = yeast_feats[_.gene_aliases |In| _.Flat().GroupBy(_.gene_aliases)[Count(_.feat_name) == 1].gene_aliases]
        convert_table = name_alias_list.Get(_.gene_aliases.Each(str.upper), _.feat_name).Flat()

        yeastract = yeastract.To(_.target, Do=_.Each(str.upper).TakeFrom(convert_table, keep_missing=True))
        tf_feat = yeastract |Match(_.target.Each(str.upper), _.feat_name.Each(str.upper))| yeast_feats

        Save(tf_feat, 'tf_feat.dat')
        tf_feat = Load('tf_feat.dat')
        
        
        tf_feat = tf_feat.GroupBy(_.trans_factor, _.chromosome)
        res = tf_feat.Get(_.trans_factor, _.chromosome, _.target.Count()/"count", _.start).Copy()
        str(res)

        str(Corr(res.count))

        str(Corr(res.count.Cast("real64") / res.count.Sum("gtrans_factor").count))
        chr_normtf = res.To(_.count, Do=_.Cast("real64") / _.count.Sum("gchromosome"))

        str(Corr(chr_normtf.count.Transpose()))


        str(chr_normtf.Sort(_.chromosome.Cast("int?")).Get(_.chromosome, Corr(_.count.Transpose()/"chromo_corr")))












        











from thirdparty.spark import GenericScanner, GenericParser

class Token(object):#{{{
    def __init__(self, type, attr=None):
        self.type = type
        self.attr = attr

    def __cmp__(self,o):
        return cmp(self.type,o)
    
    def __getitem__(self,pos):
        raise IndexError

    def __len__(self):
        return 0

    def __repr__(self):
        if(self.attr is None):
            return str(self.type)
        else:
            return str(self.type) + ":" + str(self.attr)[:20]#}}}

class AST(object):#{{{
    def __init__(self, type, kids=tuple()):
        self.type = type
        self.kids = kids

    def __getitem__(self,pos):
        return self.kids[pos]

    def __len__(self):
        return len(self.kids)

    def __repr__(self):
        return str(self.type) + str(self.kids)#}}}

    def __cmp__(self,o):
        return cmp(self.type,o)


class GenericASTRewriter:#{{{
    def typestring(self, node):
        return node.type

    def preorder(self, node=None):
        if not isinstance(node, (AST,Token)):
            return node
        name = 'n_' + self.typestring(node)
        if hasattr(self, name):
            func = getattr(self, name)
            node = func(node)
        else:
            node = self.default(node)

        node.kids = [self.preorder(kid) for kid in node]

        name = name + '_exit'
        if hasattr(self, name):
            func = getattr(self, name)
            node = func(node)
        return node

    def postorder(self, node, context=None):
        if not isinstance(node, (AST,Token)):
            return node
        name = 'x_' + self.typestring(node)
        if hasattr(self, name):
            func = getattr(self, name)
            node = func(node, context)
            return node

        node.kids = [self.postorder(kid,context) for kid in node]

        name = 'n_' + self.typestring(node)
        if hasattr(self, name):
            func = getattr(self, name)
            if context is None:
                node = func(node)
            else:
                node = func(node, context)
        else:
            node = self.default(node)
        return node

    def default(self, node):
        return node#}}}




"""Module implementing loading/writing of configuration 
variables to ini file

Author: Marc Hulsman
Date: 20-feb-08"""

import ConfigParser
import string
import os
import os.path

def loadConfig(rc_path, config={}):
    """
    returns a dictionary with key's of the form
    <section>.<option> and the values 
    """
    config = config.copy()
    if(os.path.isfile(rc_path)):    
        cp = ConfigParser.ConfigParser()
        cp.read(rc_path)
        for sec in cp.sections():
            name = string.lower(sec)
            for opt in cp.options(sec):
                config[name + "." + string.lower(opt)] = string.strip(cp.get(sec, opt))
    return config

def writeConfig(filename, config):
    """
    given a dictionary with key's of the form 'section.option: value'
    write() generates a list of unique section names
    creates sections based that list
    use config.set to add entries to each section
    """
    cp = ConfigParser.ConfigParser()
    sections = set([k.split('.')[0] for k in config.keys()])
    map(cp.add_section, sections)
    for k,v in config.items():
        s, o = k.split('.')
        cp.set(s, o, v)
    cp.write(open(filename, "w"))

''' 
When more than one value is needed from a config file in a script, please use the loadConfig method.
This method is just a shortcut when only one specific value is of interest.
'''
def getValue(file, config, value):
    cfg = loadConfig(file, config)
    return cfg[value]


_ConfigDefault = {
    "system.tmp_dir":"/tmp"
}

config = loadConfig(os.path.expanduser('~/.ibidas/ibidas.cfg'), _ConfigDefault)



import operator
from collections import defaultdict, Iterable
from ..constants import *
import rtypes
from ..utils import util

in_type_casts = defaultdict(list)
cast_exec = defaultdict(dict)

class CheckEnv(object):
    """Object holding attributes to check cast"""
    __slots__ = ['out_type_cls', 'checktypesfunc', 'simtypefunc', 'name']
    def __init__(self, out_type_cls, checktypesfunc, simtypefunc, name):
        self.out_type_cls = out_type_cls
        self.checktypesfunc = checktypesfunc
        self.simtypefunc = simtypefunc
        self.name = name

def addCasts(in_type_cls, out_type_cls, checktypesfunc, simtypefunc, castname):
    """Add cast
        in_type_cls: in type class, can also be sequence
        out_type_cls: out type class, can also be sequence
        checktypefunc: function to check actual types for compatibility
        simtypefunc: function to create best matching out_type if possible
    """
    if(isinstance(out_type_cls, Iterable)):
        if(not isinstance(out_type_cls, set)):
            out_type_cls = set(out_type_cls)
    else:
        out_type_cls = set([out_type_cls])
    
    checkenv = CheckEnv(out_type_cls, checktypesfunc, simtypefunc, castname) 
    
    if(isinstance(in_type_cls, Iterable)):
        for incls in in_type_cls:
            in_type_casts[incls].append(checkenv)
    else:
        in_type_casts[in_type_cls].append(checkenv)

def findImplicitCastTypes(in_type_cls):
    """Returns type classes that can be casted to without loss of information"""
    res = set(in_type_cls.__mro__[1:])
    res.add(in_type_cls)
    return res 

def canCast(intype, outtype, dimdepth):
    if(not intype.__class__ in in_type_casts):
        return False
    
    pos_casts = in_type_casts[intype.__class__]

    if(isinstance(outtype, rtypes.TypeUnknown)):
        for pos_cast in pos_casts:
            if(outtype.__class__ in pos_cast.out_type_cls and
               pos_cast.checktypesfunc(intype, outtype)):
                return pos_cast.name
    else:
        for pos_cast in pos_casts:        
            if(outtype in pos_cast.out_type_cls):
                otype = pos_cast.simtypefunc(intype, outtype, dimdepth)
                return (otype, pos_cast.name)
    return False

def castImplicitCommonType(type1, type2):
    if(type1 == type2):
        return type1

    in1_impli_cls = findImplicitCastTypes(type1.__class__)
    in2_impli_cls = findImplicitCastTypes(type2.__class__)
    out_impli_cls = in1_impli_cls & in2_impli_cls

    while out_impli_cls:
        out_cls = rtypes.mostSpecializedTypesCls(out_impli_cls)
        assert len(out_cls) == 1, \
            "Multiple implicit common types found"
        out_cls = out_cls[0]
        res = out_cls.commonType(type1, type2)
        if(not res is False):
            break
        out_impli_cls.discard(out_cls)
    return res

def castMultipleImplicitCommonType(*types):
    assert types, "No types given to cast function"
    types = list(types)
    while(len(types) > 1):
        types.append(castImplicitCommonType(types.pop(),types.pop()))
    return types[0]

############
# CAST: check type functions
############
def checkDefault(intype, outtype):#{{{
    return True

def simDefault(intype, outtypecls, dimdepth):
    return outtypecls(intype.has_missing)

def simSame(intype, outtypecls, dimdepth):
    return outtypecls(intype.has_missing, dims=intype.dims)


#}}}
addCasts(rtypes.TypeNumbers | set([rtypes.TypeAny, rtypes.TypeUnknown]), rtypes.TypeNumbers, checkDefault, simDefault,"to_numbers")
addCasts(rtypes.TypeAll, set([rtypes.TypeAny, rtypes.TypeUnknown]), checkDefault, simDefault,"to_any")
addCasts(set([rtypes.TypeUnknown, rtypes.TypeAny]) | rtypes.TypeStrings, rtypes.TypeIntegers, checkDefault, simDefault,"string_to_int")
addCasts(set([rtypes.TypeUnknown, rtypes.TypeAny]) | rtypes.TypeStrings, rtypes.TypeReals, checkDefault, simDefault,"string_to_real")
addCasts(set([rtypes.TypeUnknown, rtypes.TypeAny]) | rtypes.TypeStrings, rtypes.TypeStrings, checkDefault, simDefault,"string_to_string")


def checkStringString(intype, outtype):#{{{
    if(intype.has_missing and not outtype.has_missing):
        return False

    if not intype.dims[0].shape == UNDEFINED and not outtype.dims[0].shape == UNDEFINED:
        return False
    return True

def simStringString(intype, outtypecls, dimdepth):
    return intype

addCasts([rtypes.TypeBytes,rtypes.TypePickle], [rtypes.TypeBytes,rtypes.TypePickle], checkStringString, simStringString,"string_to_string")#}}}

def checkPickle(intype, outtype):#{{{
    if(intype.has_missing and not outtype.has_missing):
        return False

    if not outtype.dims[0].shape == UNDEFINED:
        return False
    return True

def simPickle(intype, outtypecls, dimdepth):
    ndim = dimensions.Dim(UNDEFINED, (True,) * dimdepth, intype.has_missing)
    return rtypes.TypePickle(dims=dimpaths.DimPath(ndim))

addCasts(rtypes.TypeAll, rtypes.TypePickle, checkPickle, simPickle,"to_pickle")

def simDePickle(intype, outtypecls, dimdepth):
    return rtypes.unknown

addCasts(rtypes.TypePickle, rtypes.TypeAll, checkDefault, simDePickle,"from_pickle")


def checkArray(intype, outtype):#{{{
    if(intype.has_missing and not outtype.has_missing):
        return False
    
    if len(outtype.dims) != 1:
        return False

    if not intype.dims[0].shape == outtype.dims[0].shape or not intype.dims[0].dependent == outtype.dims[0].dependent:
        return False

    if not intype.subtypes == outtype.subtypes:
        return False
    return True

def simArray(intype, outtypecls, dimdepth):
    return rtypes.TypeArray(has_missing=intype.has_missing, dims=intype.dims, subtypes=intype.subtypes)

addCasts(rtypes.TypeStrings | set([rtypes.TypeSet]), rtypes.TypeArray, checkArray, simArray,"to_array")
    

def checkBytes(intype, outtype):
    if(not isinstance(outtype,rtypes.TypeBytes)):
        return False
    return True

def simBytes(intype, outtypecls, dimdepth):
    ndim = dimensions.Dim(UNDEFINED, (True,) * dimdepth, intype.has_missing)
    return types.TypeBytes(dims=dimpaths.DimPath(ndim),has_missing=intype.has_missing)

addCasts(rtypes.TypeAll, rtypes.TypeBytes, checkBytes, simBytes,"to_bytes")




from .. import representor
from ..passes import manager

class Indicator(manager.Pass):
    def __eq__(self, other):
        return self.__class__ is other.__class__
    
    def __hash__(self):
        return hash(self.__class__)

class Executor(manager.Pass):
    pass


class SourceRepresentor(representor.Representor):
    pass

def open(func, *args, **kwds):
    return func(*args, **kwds)



import numpy
import copy

_delay_import_(globals(),"..utils","cutils","sparse_arrays","util")
_delay_import_(globals(),"..utils.missing","Missing")
_delay_import_(globals(),"..itypes","dimpaths","rtypes","dimensions")
class NestedArray(object):
    def __init__(self,data,cur_type):
        self.data = util.darray([data],object)
        self.cur_type = cur_type
        self.idxs = [0]
        
    def copy(self):
        res = copy.copy(self)
        res.data = self.data.view(numpy.ndarray)
        res.idxs = list(self.idxs)
        return res
  
    def getDimShape(self,pos):
        pos += 1
        if(isinstance(self.idxs[pos],int)):
            nextpos,nextobj = self._get_next_obj(pos)
            return nextobj.shape[self.idxs[pos]]
        else:
            d = self.idxs[pos]
            d = d[...,1] - d[...,0]
            return d

    def _curIdxDepth(self):
        if(isinstance(self.idxs[-1],int)):
            return self.idxs[-1] + 1
        else:
            return 1

    def flat(self):
        data,rshape = self._flatData()
        data= util.darray([subelem for subelem in data],object,1,1)
        return data

    def _flatData(self,depth=-1):
        seq = self.data
        if(isinstance(self.idxs[depth],int)):
            rshape = seq.shape[:(self.idxs[depth]+1)]
            seq = dimpaths.flatFirstDims(seq,self.idxs[depth])
        else:
            #assert depth == -1 or depth==(len(self.idxs)-1),"Cannot flat data on non-fixed dim"
            rshape = (len(seq),)
        return (seq,rshape)

    def unpack(self, dimpath, subtype):
        nself = self.copy()
        #init
        data = nself.data
        depth = len(dimpath)
        odimpath = dimpath
        while(depth>0):        
            idxdepth = nself._curIdxDepth()
            rem_dims = len(data.shape) - idxdepth
            #does cur data have dimensions left that can be unpacked?
            if(rem_dims):
                rem_depth = min(rem_dims,depth)
                nself.idxs.extend(range(idxdepth, idxdepth + rem_depth))
                dimpath = dimpath[rem_depth:]
                depth -= rem_depth
                
                if(depth == 0): #if depth is 0 we can stop here
                    break
            
            #bummer, now it gets a bit more complicated
            cdata = data.ravel()
            #determine the number of fixed dimensions that can be unpacked
            #finds number of contiguous fixed dims from start of dimpath
            #This allows to unpack a multi-dimensional part of the data
            tot_dimpath = dimpath + dimpaths.getArrayDimPathFromType(subtype)
            cdepth = tot_dimpath.contigiousFixedNDims()
            
            #prepare fixed (multi-)dimensions and variable scenario
            if(not cdepth): 
                variable = True
                idxres = numpy.zeros((len(cdata),) + (2,),dtype=int)
                curpos = 0
                if(tot_dimpath[0].has_missing):
                    cdepth = 1
                else:
                    cdepth = 1 + tot_dimpath[1:].contigiousFixedNDims()
            else:
                variable = False
            
            if(len(dimpath) > cdepth):
                dtype = object
            elif(len(dimpath) < cdepth):
                dtype = dimpaths.getNestedArraySubType(subtype, cdepth - len(dimpath)).toNumpy()
            else:
                dtype = subtype.toNumpy()


            if(variable):
                res = []
                for pos in xrange(len(cdata)):
                    elem = cdata[pos]
                    if(variable):
                        if(elem is Missing):
                            idxres[pos,:] = -1
                            continue
                        idxres[pos,0] = curpos
                        curpos = curpos + len(elem)
                        idxres[pos,1] = curpos
                
                    #check that elem shape is not smaller or larger than expected 
                    if(not isinstance(elem,numpy.ndarray)):
                        elem = util.darray(list(elem),dtype,cdepth,cdepth)
                    else:
                        elem = validate_array(elem,cdepth,dtype)
                        assert len(elem.shape) == cdepth, "Number of dimensions incorrect"
                    res.append(elem)
                if not res:
                    ndata = util.darray([])
                else:
                    ndata = numpy.concatenate(res)
            else:
                #if(cdepth == 1):
                #    r = []
                #    for elem in cdata:
                #        r.extend(elem)
                #    ndata = util.darray(r,dtype,1,1)
                #else:
                res = []
                for elem in cdata:
                    #check that elem shape is not smaller or larger than expected 
                    if(not isinstance(elem,numpy.ndarray)):
                        elem = util.darray(list(elem),dtype,cdepth,cdepth)
                    else:
                        elem = validate_array(elem,cdepth,dtype)
                    res.append(elem)
                if not res:
                    ndata = util.darray([])
                else:
                    ndata = numpy.concatenate(res)


            if(variable):
                idxres.shape =  data.shape + (2,)
                nself.idxs.append(idxres)
            else:
                assert (len(cdata) == 0) or ((ndata.shape[0] % len(cdata)) == 0), "Leftover elements in joining dimensions"
                if len(cdata):
                    pshape = ndata.shape[0] / len(cdata)
                else:
                    pshape = 0
                ndata.shape = data.shape + (pshape,) + ndata.shape[1:]
                nself.idxs.append(len(data.shape))
            data = ndata    
            depth -= 1
            dimpath = dimpath[1:]
        
        nself.cur_type = subtype
        nself.data = data
        return nself
    
    def pack(self, subtype, depth=1):
        #init
        nself = self.copy()
      
        while(depth):
            assert len(nself.idxs) > 1, "Pack operation on nestedarray without index?!"
            idx = nself.idxs.pop()

            if(not isinstance(idx,int)): #refers to fixed dim in data
                res = []
                data = nself.data
                fidx = dimpaths.flatFirstDims(idx,len(idx.shape)-2)
                for pos in xrange(len(fidx)):
                     start = fidx[pos,0]
                     stop = fidx[pos,1]
                     if(start == -1):
                        res.append(Missing)
                     else:
                        res.append(data[start:stop])
                nself.data = util.darray(res,object)
                nself.data.shape = idx.shape[:-1]
            depth -= 1
        nself.cur_type = subtype
        return nself                
 
    def getStructuredData(self):
        return self.pack(rtypes.unknown, len(self.idxs)-1).data[0]

    def map(self, func, *args, **kwargs):
        restype= kwargs.get("res_type")
        dtype = restype.toNumpy()

        if(self.cur_type.has_missing):
            def wrapfunc(seq, *args, **kwargs):
                res = []
                for elem in seq:
                    if(elem is Missing):
                        res.append(Missing)
                    else:
                        res.append(func(elem, *args, **kwargs))
                nseq = util.darray(res,dtype)
                return nseq
        else:
            def wrapfunc(seq, *args, **kwargs):
                res = [func(elem, *args, **kwargs) for elem in seq]
                nseq = util.darray(res,dtype)
                return nseq
            
        return self.mapseq(wrapfunc, *args, **kwargs)

    def mapseq(self,func,*args,**kwargs):
        restype= kwargs.pop("res_type")
        
        seq,rshape = self._flatData()
        seq = func(seq,*args,**kwargs)
        seq.shape = rshape + seq.shape[1:]

        nself = self.copy()
        nself.data = seq
        nself.cur_type = restype
        return nself

    def insertDim(self,matchpoint):
        matchpoint += 1
        nself = self.copy()
        idxs = nself.idxs + [nself.data]
        curidx = idxs[matchpoint-1]
        if(isinstance(curidx,int)):
            newidx = curidx+1
        else:
            newidx = 1
        nself.idxs.insert(matchpoint,newidx)
        for pos in range(matchpoint+1,len(nself.idxs)):
            tidx = nself.idxs[pos]
            if(isinstance(tidx,int)):
                nself.idxs[pos] += 1
            else:
                tidx = numpy.array(tidx)
                tidx.shape = tidx.shape[:newidx] + (1,) + tidx.shape[newidx:]
                nself.idxs[pos] = tidx
                break
        else:
            nself.data.shape = nself.data.shape[:newidx] + (1,) + nself.data.shape[newidx:]
        return nself

    def mergeAllDims(self):
        nself = self.copy()
        nself.idxs = [0,1]
        nself.data = self._flatData()[0]
        nself.data.shape = (1,) + nself.data.shape
        return nself
        
    def _diffToIdx(self,diff):
        nlidx = numpy.zeros(diff.shape + (2,),dtype=diff.dtype)
        nlidx[...,-1] = numpy.reshape(numpy.cumsum(diff.ravel()),diff.shape)
        nlidx[...,0] = nlidx[...,-1] - diff
        return nlidx


    def splitDim(self,matchpoint,lshape,rshape=None):
        
        #info on current dim
        cshape = self.getDimShape(matchpoint)
        matchpoint += 1
        cidx = self.idxs[matchpoint]
        
        nself = self.copy()
        nextpos,nextobj = nself._get_next_obj(matchpoint)

        if(rshape is None):
            if(isinstance(lshape,int)):
                if isinstance(cshape,int):
                    assert cshape % lshape == 0, "Cannot find matching right shape"
                    rshape = cshape / lshape
                else:
                    assert (cshape % lshape == 0).all(), "Cannot find matching right shape"
                    rshape = cshape / lshape
            else:
                raise RuntimeError, "Cannot determine right shape"

        if(isinstance(cidx,int)):
            if(isinstance(lshape,int)):
                if(isinstance(rshape,int)):
                    xshape = list(nextobj.shape)
                    assert lshape * rshape == xshape[cidx], "Splitted dimensions do not match size original dimension"
                    xshape[cidx] = rshape
                    xshape.insert(cidx,lshape)
                    nextobj = numpy.reshape(nextobj,xshape)
                    nself.idxs.insert(matchpoint,cidx)
                    for i in range(matchpoint+1,nextpos+1):
                        nself.idxs[i] += 1
                    nself._set_obj(nextpos+1,nextobj)
                else:
                    rshape = numpy.reshape(rshape,nextobj.shape[:cidx] + (lshape,))
                    assert (numpy.sum(rshape,axis=-1) == nextobj.shape[cidx]).all(), "Variable dimension lengths of rshape unequal to current shape"
                    rshape = self._diffToIdx(rshape)
                    nself.idxs[matchpoint] = rshape 
                    nself.idxs.insert(matchpoint,len(rshape.shape) - 2)
                    nextobj = dimpaths.flatFirstDims(nextobj,cidx)
                    for pos,i in enumerate(range(matchpoint+2,nextpos+1)):
                        nself.idxs[i] += pos
                    nself._set_obj(nextpos+1,nextobj)
            else:           
                #FIXME
                raise RuntimeError,"Splitting fixed to variable dimension not yet supported"
        else:
            if(isinstance(lshape,int)):
                if(isinstance(rshape,int)):
                    assert (lshape * rshape == nextobj.shape[0]), "Splitted dimensions do not match size original dimensions"
                    nextobj = nextobj.reshape(cidx.shape[:-1] + (lshape,rshape) + nextobj.shape[1:])
                    if(isinstance(nself.idxs[matchpoint-1],int)):
                        spos = nself.idxs[matchpoint-1] + 1
                    else:
                        spos = 0
                    nself.idxs[matchpoint] = spos+1
                    nself.idxs.insert(matchpoint,spos)
                    for i in range(matchpoint + 2, nextpos+1):
                        nself.idxs[i] += spos + 1
                    nself._set_obj(nextpos+1,nextobj)
                else:
                    assert nextobj.shape[0] == numpy.sum(rshape), "Splitted dimensions do not match size original dimension"
                    rshape = numpy.reshape(rshape,cidx.shape[:-1] + (lshape,))
                    rshape = self._diffToIdx(rshape)
                    nself.idxs[matchpoint] = rshape 
                    nself.idxs.insert(matchpoint,len(rshape.shape) - 2)
            else:
                if(isinstance(rshape,int)):
                    assert nextobj.shape[0] == numpy.sum(lshape) * rshape, "Splitted dimensions do not match size original dimension"
                    lshape = numpy.reshape(lshape,cidx.shape[:-1])
                    lshape = self._diffToIdx(lshape)
                    nextobj = nextobj.reshape((nextobj.shape[0] / rshape,rshape) + nextobj.shape[1:])
                    nself.idxs[matchpoint] = 0
                    nself.idxs.insert(matchpoint,lshape)
                    for i in range(matchpoint+2,nextpos+1):
                        nself.idxs[i] += 1
                    nself._set_obj(nextpos+1,nextobj)
                else:
                    rshape = rshape.ravel()
                    assert numpy.sum(lshape) == rshape.shape[0],"Splitted dimensions do not match size original dimension"
                    assert nextobj.shape[0] == numpy.sum(rshape), "Splitted dimensions do not match size original dimension"
                    lshape = numpy.reshape(lshape,cidx.shape[:-1])
                    lshape = self._diffToIdx(lshape)
                    rshape = self._diffToIdx(rshape)
                    nself.idxs[matchpoint] = rshape
                    nself.idxs.insert(matchpoint,lshape)
        return nself

    def mergeDim(self, matchpoint, result_fixed=True):
        matchpoint += 1
        assert len(self.idxs) > (matchpoint + 1), "Nested array not nested that deep"
        nself = self.copy()
        lidx = self.idxs[matchpoint]
        ridx = self.idxs[matchpoint+1]
        #Merging dims, 4 possibilities:
        #fixed-fixed, fixed-var, var-fixed, var-var
        if(isinstance(ridx,int)):
            nextpos,nextobj = nself._get_next_obj(matchpoint+1)
            oshape = nextobj.shape
            if(isinstance(lidx,int)):  #fixed-fixed
                nshape = oshape[:lidx] + (oshape[lidx] * oshape[ridx],) + oshape[(ridx+1):]
            else:  #var-fixed
                diff = (lidx[...,-1] - lidx[...,0]) * oshape[ridx]
                nlidx = numpy.zeros(lidx.shape,dtype=lidx.dtype)
                nlidx[...,-1] = numpy.reshape(numpy.cumsum(diff.ravel()),diff.shape)
                nlidx[...,0] = nlidx[...,-1] - diff
                nself.idxs[matchpoint] = nlidx
                nshape =  (oshape[0] * oshape[ridx],) + oshape[(ridx+1):]
            for i in range(matchpoint + 1, nextpos):
                nself.idxs[i] = nself.idxs[i] -1
            nextobj = nextobj.reshape(nshape)
            nself._set_obj(nextpos,nextobj)
        else:
            if(isinstance(lidx,int)):  #fixed-var
                oshape = ridx.shape
                nshape = oshape[:-2] + (2 * oshape[-2],)
                nridx = ridx.reshape(nshape)[...,[0, -1]]
                nself.idxs[matchpoint] = nridx
            else:     #var-var
                nlidx = numpy.zeros(lidx.shape,dtype=lidx.dtype)
                if(len(ridx) > 0):
                    nlidx[...,0] = ridx[lidx[...,0],0]
                    nlidx[...,-1] = numpy.roll(nlidx[...,0],-1)
                    nlidx.ravel()[-1] = ridx.ravel()[-1]
                nself.idxs[matchpoint] = nlidx
        del nself.idxs[matchpoint+1]
        if(not isinstance(nself.idxs[matchpoint],int) and result_fixed):
            nself._var_to_fixed(matchpoint), 
        return nself

    def _var_to_fixed(self, pos):
        idx = self.idxs[pos]
        nextpos,nextobj = self._get_next_obj(pos)
        diff = idx[...,-1] - idx[...,0]
        diffset = set(diff.ravel())

        assert len(diffset)==1, "Multiple lengths encountered while expecting fixed dim"
        
        nshape = diffset.pop()
        nextobj = nextobj.reshape(diff.shape + (nshape,) + nextobj.shape[1:])

        if(pos > 0 and isinstance(self.idxs[pos-1],int)):
            prevpos = self.idxs[pos-1]
        else:
            prevpos = 0
            
        for i in range(pos, nextpos):
            self.idxs[i] = (i - pos) + prevpos + 1
        self._set_obj(nextpos,nextobj)

    def mergeLastDims(self,depth):
        curdepth = len(self.idxs) - 1
        assert depth < curdepth, "Attempted merge to deep"
        
        nself = self
        nshapes = []
        for d in range(curdepth - depth, curdepth)[::-1]:
            s1 = nself.getDimShape(d-1)
            s2 = nself.getDimShape(d)
            nshapes.append((s1,s2))
            nself = nself.mergeDim(d-1, True)
        return nself, nshapes[::-1]

    def splitLastDim(self, shapes):
        nself = self
        for shapeleft, shaperight in shapes:
            nself = nself.splitDim(len(nself.idxs) - 2, shapeleft,shaperight)
        return nself

    def swapDim(self, matchpoint):
        matchpoint += 1
        assert len(self.idxs) > (matchpoint + 1), "Nested array not nested that deep"
        nself = self.copy()
        nself._swapDim(matchpoint)
        return nself

    def _swapDim(self, matchpoint):#{{{
        nself = self
        lidx = self.idxs[matchpoint]
        ridx = self.idxs[matchpoint+1]
        #Merging dims, 4 possibilities:
        #fixed-fixed, fixed-var, var-fixed, var-var
        if(isinstance(lidx,int)):
            nextpos,nextobj = nself._get_next_obj(matchpoint+1)
            if(isinstance(ridx,int)):  #fixed-fixed
                pidx = range(len(nextobj.shape))
                tmp = pidx[lidx]
                pidx[lidx] = pidx[ridx]
                pidx[ridx] = tmp
                #added copy to ensure that array is c-contiguous,
                #as not being contiguous kills performance
                #In the future, maybe we should move contigious checks
                #to the operations that benefit from them, then we 
                #can remove this copy (and the copy below in var-var)
                nextobj = numpy.transpose(nextobj,axes=pidx).copy()
            else: #fixed-var --> var-fixed
                fixshape = ridx.shape[-2]
                nnextobj = numpy.zeros(nextobj.shape,dtype=nextobj.dtype)
                nnextobj = numpy.reshape(nnextobj, (nextobj.shape[0] / fixshape,fixshape) + nextobj.shape[1:])

                source_ridx = dimpaths.flatFirstDims(ridx,len(ridx.shape) -2).copy()
                
                ridx = ridx[...,0,:]
                oshape = ridx.shape
                diff = ridx[...,1] - ridx[...,0]
                ridx[...,1] = numpy.reshape(numpy.cumsum(diff.ravel()),diff.shape)
                ridx[...,0] = ridx[...,1] - diff

                dest_ridx = dimpaths.flatFirstDims(ridx,len(ridx.shape)-2)
                range_fixshape = range(fixshape)
                for i in range(0,len(dest_ridx)):
                    dest = slice(dest_ridx[i,0],dest_ridx[i,1])
                    for j in range_fixshape:
                        xj = j + i * fixshape
                        nnextobj[dest,j] = nextobj[source_ridx[xj,0]:source_ridx[xj,1]]
                nextobj = nnextobj

                nself.idxs[matchpoint] = ridx
                for i in range(matchpoint + 1, nextpos):
                    nself.idxs[i] = i -(matchpoint + 1) + 1
            nself._set_obj(nextpos,nextobj)
            nself = nself._normalize(nextpos)
        else:
            nextpos,nextobj = nself._get_next_obj(matchpoint+1)
            if(isinstance(ridx,int)): #var-fixed --> fixed-var
                fixshape = nextobj.shape[1]
                
                nnextobj = numpy.zeros(nextobj.shape,dtype=nextobj.dtype)
                nnextobj = dimpaths.flatFirstDims(nnextobj,1)

                source_lidx = dimpaths.flatFirstDims(lidx,len(lidx.shape)-2)
                
                diff = source_lidx[:,1] - source_lidx[:,0]
                diff = numpy.repeat(diff,fixshape)
                nlidx = numpy.zeros((len(diff),2),dtype=int)
                nlidx[:,1] = numpy.cumsum(diff)
                nlidx[:,0] = nlidx[:,1] -diff
                nlidx.shape = lidx.shape[:-1] + (fixshape,2)

                dest_lidx = dimpaths.flatFirstDims(nlidx,len(nlidx.shape)-2)
                range_fixshape = range(fixshape)
                for i in range(0,len(source_lidx)):
                    src = slice(source_lidx[i,0],source_lidx[i,1])
                    for j in range_fixshape:
                        xj = j + i * fixshape
                        nnextobj[dest_lidx[xj,0]:dest_lidx[xj,1]] = nextobj[src,j]
                nextobj=nnextobj
                
                nself.idxs[matchpoint] = len(nlidx.shape) - 2
                nself.idxs[matchpoint+1] = nlidx

                for i in range(matchpoint +2, nextpos):
                    nself.idxs[i] = i - (matchpoint + 1)
            else: #var-var --> var-var
                nnextobj = numpy.zeros(nextobj.shape,dtype=nextobj.dtype)
                source_lidx = dimpaths.flatFirstDims(lidx,len(lidx.shape)-2)
                source_ridx = dimpaths.flatFirstDims(ridx,len(ridx.shape)-2)

                ldiff = source_lidx[:,1] - source_lidx[:,0]
                rdiff = source_ridx[source_lidx[:,0],1] - source_ridx[source_lidx[:,0],0]
                repeat = rdiff.copy()
                ldiff[repeat==0] = 0
                repeat[repeat==0] =1
                ldiff = numpy.repeat(ldiff,repeat)

                nlidx = numpy.zeros((len(ldiff),2),dtype=int)
                nlidx[:,1] = numpy.cumsum(ldiff)
                nlidx[:,0] = nlidx[:,1] -ldiff

                nridx = numpy.zeros((len(rdiff),2),dtype=int)
                nridx[:,1] = numpy.cumsum(rdiff)
                nridx[:,0] = nridx[:,1] -rdiff
                nridx.shape = lidx.shape
              
                pidx = range(len(nextobj.shape)+1)
                pidx[0] = 1
                pidx[1] = 0
                for i in range(0,len(source_lidx)):
                    lsrc = slice(source_lidx[i,0],source_lidx[i,1])
                    rsrc = slice(source_ridx[lsrc.start,0],source_ridx[lsrc.stop-1,1])
                    leftlen = lsrc.stop - lsrc.start
                    if(leftlen == 0):
                        rightlen= (rsrc.stop - rsrc.start)
                    else:
                        rightlen= (rsrc.stop - rsrc.start) / leftlen

                    d = numpy.reshape(nextobj[rsrc],(leftlen,rightlen) +  nextobj.shape[1:])
                    d = numpy.transpose(d,pidx).copy()
                    nnextobj[rsrc] = d.ravel()
                nextobj = nnextobj

                nself.idxs[matchpoint] = nridx
                nself.idxs[matchpoint+1] = nlidx
            nself._set_obj(nextpos,nextobj)
            nself = nself._normalize(matchpoint + 2)#}}}

    def permuteDims(self, permute_idxs):
        nself = self.copy()
        assert len(permute_idxs) == (len(self.idxs) - 1), "Permute idxs do not cover all dimensions"
      
        #determine new dim positions
        dim_permute = [0] * len(permute_idxs)
        for pos, pi in enumerate(permute_idxs):
            dim_permute[pi] = pos
        permute_idxs = dim_permute

        for i in range(len(permute_idxs)):
            for j in range(len(permute_idxs) - i - 1):
                if permute_idxs[j] > permute_idxs[j+1]:
                    nself._swapDim(j+1) #correct for first idx
                    permute_idxs[j],permute_idxs[j+1] = permute_idxs[j+1],permute_idxs[j]

        return nself

           
    def broadcast(self, repeat_dict):
        if not repeat_dict:
            return self
        nself = self.copy()

        repeats = [1] * len(self.idxs)
        for pos,repeat  in repeat_dict.iteritems():
            repeats[pos + 1] = repeat
        tilerepeats = []
        prev_repeat = False
        for pos,idx,repeat in zip(range(len(repeats)),nself.idxs,repeats):
            if(isinstance(idx,int)):
                if(isinstance(repeat,int)):
                    tilerepeats.append(repeat)
                else:
                    nextpos,nextobj = nself._get_next_obj(pos)
                    nself._apply_tile_broadcast(tilerepeats,nextpos,prev_repeat)
                    nextpos,nextobj = nself._get_next_obj(pos)
                    
                    #perform some checks
                    ntr = len(tilerepeats)
                    assert nextobj.shape[ntr] == 1, "Varbroadcast on full dimension not possible"
                    bcshape = list(nextobj.shape[:(ntr - len(repeat.shape))])
                    for l,d in zip(repeat.shape,nextobj.shape[(ntr-len(repeat.shape)):ntr]):
                        if(l != d):
                            assert l == 1, "Problem in broadcasting repeat shape"
                            bcshape.append(d)
                        else:
                            bcshape.append(1)
                    repeat = numpy.tile(repeat,bcshape) 

                    #collapse first dims
                    nextobj = dimpaths.flatFirstDims(nextobj,ntr)

                    #replace new idx
                    nself.idxs[pos] = self._diffToIdx(repeat)
                  
                    #repeat nextobj
                    nextobj = numpy.repeat(nextobj,repeat.ravel(),axis=0)
                    nself._set_obj(nextpos,nextobj)

                    #update idxs 
                    for p in range(pos+1,nextpos):
                        nself.idxs[p] = p - pos

                    prev_repeat=True
                    tilerepeats = [1]
            else:
                assert repeat == 1, "Broadcast of full dimension not possible"
                nself._apply_tile_broadcast(tilerepeats,pos,prev_repeat)
                prev_repeat=True
                tilerepeats = [1]
        
        if(tilerepeats):
            nself._apply_tile_broadcast(tilerepeats,len(self.idxs),prev_repeat)
        return nself

    def _get_next_obj(self,pos):
        pos += 1
        while(len(self.idxs) > pos and isinstance(self.idxs[pos],int)):
            pos += 1
        if(pos >= len(self.idxs)):
            return (pos,self.data.copy())
        else:
            return (pos,self.idxs[pos])

    def _set_obj(self,pos,obj):
        if(pos == len(self.idxs)):
            self.data = obj
        else:
            self.idxs[pos] = obj
        
            
    def _apply_tile_broadcast(self, tilerepeats, pos, prevrepeat):
        if(not tilerepeats and not prevrepeat):
            return
        if(pos == len(self.idxs)):
            idx = self.data
        else:
            idx = self.idxs[pos]
       
        assert isinstance(idx,numpy.ndarray), "Tilerepeats should be applied to numpy idx array"
        tilerepeats = list(tilerepeats)
        while(len(tilerepeats) < len(idx.shape)):
            tilerepeats.append(1)
        
        assert len(tilerepeats) == len(idx.shape), "Number of repeats does not match shape"

        shapeok = [col == 1 for tr,col in zip(tilerepeats,idx.shape) if tr != 1]
        if(not shapeok and not prevrepeat):#nothing to broadcast
            return
      
        assert all(shapeok), "Repeat of full dimension: error in  broadcast"

        if(pos == len(self.idxs)): #apply to data
            self.data = numpy.tile(self.data,tilerepeats)
            return
         
        idx = numpy.tile(idx,tilerepeats)

        oshape = idx.shape
        if(shapeok):
            lastrepeat = 0
            for tpos in xrange(len(tilerepeats)):
                if(tilerepeats[tpos] > 1):
                    lastrepeat = tpos

            lastrepeat += 1
            nshape = (numpy.multiply.reduce(oshape[:lastrepeat]),numpy.multiply.reduce(oshape[lastrepeat:]))
            idx.shape = nshape
        else:
            idx = dimpaths.flatFirstDims(idx,len(idx.shape) -2)
        #make contigious (slices in order), affects nextobj
        nextpos, nextobj = self._get_next_obj(pos)
        lastpos = 0
        res = []
        nidx = numpy.zeros(idx.shape,dtype=idx.dtype) + idx
        for rowpos in xrange(len(idx)):
            start = idx[rowpos,0]
            stop = idx[rowpos,-1]
            res.append(nextobj[start:stop])
            nidx[rowpos,:] += lastpos - start
            lastpos += stop - start
        if not res:
            nextobj = util.darray(res)
        else:
            nextobj = numpy.concatenate(res)
        nidx.shape  = oshape
        self._set_obj(nextpos,nextobj)
        self.idxs[pos] = nidx
        
    def _normalize(self, curpos):
        if(curpos >= len(self.idxs)):
            return self

        nself = self
        curpos,curobj = self._get_next_obj(curpos-1)
        nextpos,nextobj = self._get_next_obj(curpos)
        while curpos < len(self.idxs):
            respos = 0
            res = []
            oshape = curobj.shape 
            curobj = dimpaths.flatFirstDims(curobj, len(curobj.shape) - 2)
            nidx = numpy.zeros(curobj.shape,dtype=curobj.dtype)
            for rowpos in xrange(len(curobj)):
                start = curobj[rowpos,0]
                stop = curobj[rowpos,-1]
                res.append(nextobj[start:stop])
                nidx[rowpos,0] = respos
                respos += stop - start
                nidx[rowpos,1] = respos
            if not res:
                nextobj = util.darray(res)
            else:
                nextobj = numpy.concatenate(res)
            nidx.shape  = oshape
            nself._set_obj(curpos, nidx)
            nself._set_obj(nextpos, nextobj)

            curpos,curobj = self._get_next_obj(curpos)
            nextpos,nextobj = self._get_next_obj(nextpos)
        return nself

 


    def __repr__(self):
        return "NestedArray < \n" + \
               "Idxs: " + str(self.idxs) + " Shape: " + str(self.data.shape) + "\n" + \
               "Data: " + str(self.data) + "  Dtype: " + str(self.data.dtype) + "\n" + \
               ">\n"



def co_mapseq(func, nested_arrays, *args, **kwargs):
    restype= kwargs.pop("res_type")
    
    idxlen = set([len(na.idxs) for na in nested_arrays])
    assert len(idxlen) == 1, "Nested arrays should have same dimensions!"
    idxlen = idxlen.pop()

    na_ref = nested_arrays[0]

    #determine if there is incomplete broadcasting going on
    lastpos = len(na_ref.idxs)-1
    lasti = 0
    xshape = []
    for i in range(idxlen):
        pos = idxlen -i - 1

        nested = [not isinstance(na.idxs[pos],int) for na in nested_arrays]
        if all(nested):
            break
        dshapes = [na.getDimShape(pos-1) for na in nested_arrays]
        if any(nested):
            for dpos in xrange(len(dshapes)):
               if not isinstance(dshapes[dpos],int):
                  red = set(dshapes[dpos])
                  assert len(red) == 1, "Unequal dims (mixed nested/non-nested) in co_mapseq"
                  nested_arrays[dpos]._var_to_fixed(pos)
                  dshapes[dpos] = nested_arrays[dpos].getDimShape(pos-1)
                  assert isinstance(dshapes[dpos],int), "After conversion to fixed dim still nested dim"
          
        res = set(dshapes)
        if len(res) > 1:
            assert len(res) == 2 and 1 in res, "Unequal dims in co_mapseq"
            lastpos = pos - 1
            lasti = i
            res.discard(1)
            xshape.append(res.pop())
        else:
            xshape.append(res.pop())
    xshape = xshape[:(lasti+1)]
    bcdepth = len(na_ref.idxs) - lastpos

    dummy,flatshape = na_ref._flatData(depth=lastpos)
    data = []
    for na in nested_arrays:
        seq,dummy = na._flatData(depth=lastpos)
        if(bcdepth > 1 and len(seq.shape) > bcdepth): #if broadcasting, we want to explicitly pack data 
            nseq = dimpaths.flatFirstDims(seq,bcdepth-1)
            nseq = util.darray(list(nseq))
            nseq.shape = seq.shape[:bcdepth]
            seq = nseq
        data.append(seq)
    
    if(bcdepth > 1):
        kwargs["bcdepth"] = bcdepth
        flatshape = flatshape + tuple(xshape[::-1])

    seq = func(data,*args, **kwargs)
    if(isinstance(restype,tuple)):
        nselfs = []
        for elem,rtype in zip(seq,restype):
            elem.shape = flatshape + elem.shape[1:]
            nself = na_ref.copy()
            nself.data = elem
            nself.cur_type = rtype
            nselfs.append(nself)
        return nselfs
    else:
        seq.shape = flatshape + seq.shape[1:]
        nself = na_ref.copy()
        nself.data = seq
        nself.cur_type = restype
        return nself


def co_map(func, narrays, *args, **kwargs):
    restype= kwargs.get("res_type")
    if(isinstance(restype,tuple)):
        dtypes = [rtype.toNumpy() for rtype in restype]
        def wrapfunc(seqs, *args, **kwargs):
            if 'bcdepth' in kwargs:
                del kwargs['bcdepth']
                zipdata = numpy.broadcast(*seqs)
            else:
                zipdata = zip(*seqs)
            res = [func(elems, *args, **kwargs) for elems in zipdata]
            xres = []
            for pos,dtype in enumerate(dtypes):
                nseq = util.darray([row[pos] for row in res],dtype)
                xres.append(nseq)
            return tuple(xres)
    else: 
        dtype=restype.toNumpy()
        def wrapfunc(seqs, *args, **kwargs):
            if 'bcdepth' in kwargs:
                del kwargs['bcdepth']
                zipdata = numpy.broadcast(*seqs)
            else:
                zipdata = zip(*seqs)
            res = [func(elems, *args, **kwargs) for elems in zipdata]
            nseq = util.darray(res,dtype)
            return nseq
    return co_mapseq(wrapfunc, narrays, **kwargs)

def drop_prev_shapes_dim(ndata,shapes):
    cidx = ndata.idxs[-1]
    if(not isinstance(cidx,int)):
        cidx = 0
    sel = 0

    nshapes = []
    for lshape,rshape in shapes:
        if(isinstance(lshape,int)):
            if(not isinstance(rshape,int)):
                s = [slice(None,None)] * (cidx + 1)
                s[-1] = sel
                rshape = rshape[s]
                cidx = 0
                sel = slice(0,numpy.sum(rshape))
        else:
            s = [slice(None,None)] * (cidx + 1)
            s[-1] = sel
            lshape = lshape[s]
            cidx = 0
            sel = slice(0,numpy.sum(lshape))
            if(not isinstance(rshape,int)):
                s = [slice(None,None)] * (cidx + 1)
                s[-1] = sel
                rshape = rshape[s]
                cidx = 0
                sel = slice(0,numpy.sum(rshape))
        nshapes.append((lshape,rshape))
    return nshapes
 

def validate_array(seq, cdepth, dtype):
    if(len(seq.shape) < cdepth):
        oshape = seq.shape
        rem_ndims = cdepth - len(seq.shape) + 1
        if len(seq) == 0:
            seq = util.darray([],dtype)
            seq.shape = (0,) * rem_ndims
        else:
            seq = util.darray(list(seq.ravel()),dtype,rem_ndims,rem_ndims)
            seq.shape = oshape + seq.shape[1:]
        assert len(seq.shape) == cdepth, "Non array values encountered for dims " + str(dims[len(seq.shape):])
    elif(len(seq.shape) > cdepth):
        oshape = seq.shape
        seq = dimpaths.flatFirstDims(seq,cdepth-1)
        seq = util.darray([subelem for subelem in seq],object,1,1)
        seq.shape = oshape[:cdepth]

    if not seq.dtype == dtype:
        if(dtype.char == 'S' or dtype.char == 'U' or dtype.char == 'V'):
            z = numpy.zeros(seq.shape,dtype)
            z[:] = seq
            seq = z
        else:
            seq = numpy.cast[dtype](seq)
    return seq




import __builtin__

delayed_imports = []

def delay_import(module, name, *args):
    global delayed_imports
    delayed_imports.append((module, name, args))

def perform_delayed_imports():
    global delayed_imports
    delayed_imports = delayed_imports[::-1]
    while delayed_imports:
        module, name, args = delayed_imports.pop()
        ximport(name,module,args)


__builtin__.__dict__['_delay_import_'] = delay_import

def ximport(name, module, args=[]):
    level = 0
    while(name.startswith('.')):
        name = name[1:]
        level += 1
    
    if(level == 0):
        level = -1
       
    try: 
        if(not args):
            module[name.split('.')[0]] = __import__(name,module,module,[],level)
        elif(len(args) == 1 and args[0] == "*"):
            res = __import__(name,module,module,[],level)
            for arg in res.__dict__.keys():
                if(arg.startswith('__')):
                    continue
                try:
                    module[arg] = getattr(res,arg)
                except AttributeError,e:
                    raise RuntimeError,"importing " + str(arg) + " from " + name + " into " + module['__name__'] + " did not succeed"
        else:
            try:
                res = __import__(name,module,module,args,level)
                for arg in args:
                    try:
                        module[arg] = getattr(res,arg)
                    except AttributeError,e:
                        raise RuntimeError,"importing " + str(arg) + " from " + name + " into " + module['__name__'] + " did not succeed"
            except ValueError:
                for arg in args:
                    res = __import__(name + "." + arg,module,module,args,level)
                    module[arg] = res
    except ImportError, e:
        raise ImportError("importing from " + name + "(" + str(level) + "): " + str(args) + " into " + module["__name__"] + " did not succeed: " + e.message)



from ibidas.utils import util
from ibidas.utils.config import config
from ibidas.pre import Pre
_delay_import_(globals(),"ibidas","*")

predefined_algs = Pre()


def Whiten(source, dim=None):
    return ((source - source.Mean(dim=dim)) / source.Std(dim=dim))
predefined_algs.register(Whiten,category="scaling")



from collections import defaultdict
from ibidas.utils import util
import copy

class Graph(object):
    def __init__(self):
        self.root = None
        self.nodes = set()
        self.node_attributes = defaultdict(dict)
        self.na = self.node_attributes
        self.edge_source = defaultdict(set)
        self.edge_target = defaultdict(set)

    def checkGraph(self):
        for key,value in self.node_attributes.iteritems():
            assert set(value.keys()) <= self.nodes, "Node attribute nodes not subset of graph ndoes"

    def setRoot(self, root):
        self.root = root

    def addNode(self,node):
        assert isinstance(node,Node), "Cannot add non-node to graph"
        self.nodes.add(node)

    def addEdge(self,edge):
        assert edge.source in self.nodes, "Unknown edge source"
        assert edge.target in self.nodes, "Unknown edge target"
        self.edge_source[edge.source].add(edge)
        self.edge_target[edge.target].add(edge)

    def copyNode(self, node):
        nnode = copy.copy(node)
        self.nodes.add(nnode)
        if node in self.edge_source:
            for edge in list(self.edge_source[node]):
                self.dropEdge(edge)
                edge.source = nnode
                self.addEdge(edge)
            del self.edge_source[node]
        if node in self.edge_target:
            for edge in list(self.edge_target[node]):
                self.dropEdge(edge)
                edge.target = nnode
                self.addEdge(edge)
            del self.edge_target[node]
        self.nodes.discard(node)
        for key, value in self.node_attributes.iteritems():
            if node in value:
                value[nnode] = value[node]
                del value[node]
        return nnode

    def dropNode(self, node):
        if node in self.edge_source:
            for edge in list(self.edge_source[node]):
                self.dropEdge(edge)
            del self.edge_source[node]
        if node in self.edge_target:
            for edge in list(self.edge_target[node]):
                self.dropEdge(edge)
            del self.edge_target[node]
        self.nodes.discard(node)
        for key, value in self.node_attributes.iteritems():
            if node in value:
                del value[node]

    def dropEdge(self, edge):
        self.edge_source[edge.source].discard(edge)
        self.edge_target[edge.target].discard(edge)
   
    def getDataEdge(self, node, pos=0, name="slice"):
        for edge in self.edge_target[node]:
            if(isinstance(edge,ParamEdge) and edge.name == name):
                assert pos == 0, "Cannot have pos larger than 0"
                return edge
            elif(isinstance(edge,ParamListEdge) and edge.name == name + "s" and edge.pos == pos):
                return edge
        raise RuntimeError, "Edge with pos: " + str(pos) + " not found for node: " + str(node) + ":" + str(node.__class__.__name__)

    def getEdge(self, source, target):
        res = self.getEdges(source, target)
        assert len(res) == 1, "More than one edge found"
        return res[0]
    
    def getEdges(self, source, target):
        edges = self.edge_source[source]
        return [edge for edge in edges if edge.target == target]

    def getSources(self):
        targets = set()
        for key,value in self.edge_target.iteritems():
            if value:
                targets.add(key)
        return self.nodes - targets

    def pruneGraph(self):
        visited_nodes = set()
        visited_edges = set()
        node_queue = [self.root]
        while(node_queue):
            node = node_queue.pop()
            if(node in visited_nodes):
                continue
            visited_nodes.add(node)
            for edge in self.edge_target[node]:
                visited_edges.add(edge)
                node_queue.append(edge.source)
        
        self.nodes = self.nodes & visited_nodes
        for key, value in list(self.edge_source.iteritems()):
            r = value & visited_edges
            if not r:
                del self.edge_source[key]
            else:
                self.edge_source[key] = r
        
        for key, value in list(self.edge_target.iteritems()):
            r = value & visited_edges
            if not r:
                del self.edge_target[key]
            else:
                self.edge_target[key] = r


    def walkUseContig(self, node, include_first=True):
        return WalkContigIter(self, node, True, include_first)

    def walkSourceContig(self, node, include_first=True):
        return WalkContigIter(self, node, False, include_first)
    
    def remove_unaryop(self, node):
        source = self.getDataEdge(node).source
        target_edges = list(self.edge_source[node])
        self.dropNode(node)
        for target_edge in target_edges:
            assert isinstance(target_edge,ParamEdge), "Unknown edge type encountered"
            target_edge.source = source
            self.addEdge(target_edge)
        return source            


class WalkContigIter(object):
    def __init__(self, graph, node, walk_users, include_first=True):
        self.node = node
        self.graph = graph
        self.walk_users = walk_users
        if not include_first:
            self.next()

    def __iter__(self):
        return self

    def next(self):
        node = self.node
        if not node is None:
            if self.walk_users:
                edges = self.graph.edge_source[self.node]
                if len(edges) > 1:
                    self.node = None
                else:
                    edge = list(edges)[0]
                    self.node = edge.target
            else: #walk_source
                edges = self.graph.edge_target[self.node]
                if len(edges) > 1:
                    self.node = None
                else:
                    edge = list(edges)[0]
                    self.node = edge.source
            return node
        else:
            raise StopIteration
    

class Node(object):
    pass


class Edge(object):
    __slots__ = ["source","target","type","subtype","attr"]

    def __init__(self,source,target,type=None,subtype=None,attr=None):
        self.source = source
        self.target = target
        self.type = type
        self.subtype = subtype
        self.attr = attr

class ParamEdge(Edge):
    __slots__ = ["name"]
    def __init__(self, source, target, name):
        self.name = name
        Edge.__init__(self, source, target)

    def __str__(self):
        return self.name

class ParamListEdge(ParamEdge):
    __slots__ = ["pos"]
    
    def __init__(self, source, target, name, pos):
        self.pos = pos
        ParamEdge.__init__(self, source, target, name)

    def __str__(self):
        return str(self.pos) + ":" + self.name 

class ParamChoiceEdge(ParamEdge):
    __slots__ = []

class ParamChoiceListEdge(ParamListEdge):
    __slots__ = []



import itertools
from collections import defaultdict
import numpy

from ..constants import *
from dimensions import Dim
#_delay_import_(globals(),"dimensions","Dim")
_delay_import_(globals(),"rtypes")
_delay_import_(globals(),"..representor")
_delay_import_(globals(),"..utils","toposort","util","context")

class PathError(Exception):
    pass

class DimPath(tuple):
    def __new__(cls, *dims):
        assert all([isinstance(dim, Dim) for dim in dims]),"DimPath should consist of dimensions"
        return tuple.__new__(cls, dims)
   
    def __reduce__(self):
        return (DimPath, tuple(self))

    def _getNames(self):
        return [dim.name for dim in self]
    names = property(fget=_getNames)
    
    def _getIDs(self):
        return [dim.id for dim in self]
    ids = property(fget=_getIDs)

    def hasName(self,name):
        return any([dim.name == name for dim in self])

    def getDimByName(self,name):
        return self[self.names.index(name)]

    def getDimIndexByName(self,name):
        dimnames = self.names
        try:
            return dimnames.index(name)
        except ValueError:
            return None

    def getDimIndices(self, selector):
        if isinstance(selector,int):
            return (selector,)
        elif isinstance(selector,str):
            return (self.getDimIndexByName(selector),)
        elif isinstance(selector,tuple):
            return sum([self.getDimIndices(elem) for elem in selector],())

    def __getslice__(self,*idx):
        res = tuple.__getslice__(self,*idx)
        if(isinstance(res,tuple)):
            return DimPath(*res)
        return res

    def __getitem__(self,*idx):
        res = tuple.__getitem__(self,*idx)
        if(isinstance(res,tuple)):
            return DimPath(*res)
        return res

    def __repr__(self):
        return "DimPath" + tuple.__repr__(self)
    

    def __add__(self, other):
        return DimPath(*tuple.__add__(self,other))

    def strip(self):
        if(self and self[0] is root):
            if(self[-1] is end):
                return self[1:-1]
            else:
                return self[1:]
        else:
            if(self and self[-1] is end):
                return self[:-1]
            else:
                return self

    def __contains__(self,part):#{{{
        if(isinstance(part,DimPath)):
            if(not part):
                return False
            elif(part[0] is root):
                if(part[-1] is end):
                    return self == part[1:-1]
                else:
                    return self[:(len(part)-1)] == part[1:]
            elif(part[-1] is end):
                return self[-(len(part)-1):] == part[:-1]
            else:
                pos = 0
                ndims = len(self)
                nmpath = len(part)
                while(pos < ndims and (ndims - pos) >= nmpath):
                    try:
                        curstart = self.index(part[0], pos)
                    except ValueError:
                        return False

                    if(self[(curstart + 1):(curstart + nmpath)] == part[1:]):
                        return True
                    else:
                        pos = curstart + 1
                return False
        else:
            return tuple.__contains__(self,part)#}}}

    def changeNames(self,newname_dict):#{{{
        newpath = []
        for dim in self:
            if(dim in newname_dict):
                dim = dim.copy()
                dim.name = newname_dict[dim]
            elif(dim.name in newname_dict):
                dim = dim.copy()
                dim.name = newname_dict[dim.name]
            newpath.append(dim)
        return DimPath(*newpath)#}}}
    
    def removeDim(self, pos, elem_specifier=None, subtype=None):#{{{
        ndims = []
        for p in xrange(max(pos + 1,0), len(self)):
            r = self[p].removeDepDim(p - pos - 1, elem_specifier)
            ndims.append(r)

        res = self[:max(pos,0)] + DimPath(*ndims)
        if(not subtype is None):
            subtype = subtype._removeDepDim(pos=pos - len(self), elem_specifier=elem_specifier)
            return (res,subtype)
        else:
            return res#}}}

    def updateDim(self, pos, ndim, subtype=None):#{{{
        if(not isinstance(ndim,tuple)):
            ndim = (ndim,)
        
        ndims = []
        for p in xrange(max(pos + 1,0), len(self)):
            ndims.append(self[p].updateDepDim(p - pos - 1, ndim))
      
        if pos >=0:
            res = self[:max(pos,0)] + ndim + DimPath(*ndims)
        else:
            res = DimPath(*ndims)

        if(not subtype is None):     
            subtype = subtype._updateDepDim(pos=pos - len(self), ndim=ndim)
            return (res,subtype)
        else:
            return res#}}}
    
    def insertDim(self, pos, ndim, subtype=None):#{{{
        ndims = []
        if(pos < 0):
            refpos = pos;
        else:
            refpos = pos - 1;

        for p in xrange(max(pos,0), len(self)):
            ndims.append(self[p].insertDepDim(p - refpos - 1, ndim))
        if pos >= 0:
            res = self[:pos] + (ndim,) + DimPath(*ndims)
        else:
            res = DimPath(*ndims)
        if(not subtype is None):
            subtype = subtype._insertDepDim(pos=refpos - len(self), ndim=ndim)
            return (res,subtype)
        else:
            return res#}}}

    def permuteDims(self,permute_idxs, subtype=None, prevdims=tuple()):#{{{
        ndims = []
        if(prevdims):
            permute_start = len(permute_idxs) - len(prevdims)
            for pos, dim in enumerate(self):
                dep = dim.dependent
                if(len(dep) > pos - permute_start):
                    deppos = util.select(range(-len(prevdims),pos)[::-1],dep)
                    ndep = [False] * (len(prevdims) + pos)
                    redim=False
                    for dp in deppos:
                        if dp < permute_start:
                            ndep[permute_idx.index(dp + len(prevdims))] = True
                            redim=True
                        else:
                            ndep[dp + len(prevdims)] = True
                    if(redim):
                        dim = dim.changeDependent(ndep[::-1],(prevdims + tuple(ndims))[::-1])
                ndims.append(dim)
        else:
            for pos, permute_idx in enumerate(permute_idxs):
                dim = self[permute_idx]
                dep = dim.dependent
                if(dep):
                    deppos = util.select(range(0,permute_idx)[::-1],dep)
                    ndep = [False] * pos
                    for dp in deppos:
                        assert dp in permute_idxs[:pos], "Dependent dim " + str(self[dp]) + " of dim " + str(dim) + " cannot be placed after it"
                        ndep[permute_idxs.index(dp)]=True
                    dim = dim.changeDependent(ndep[::-1],ndims[::-1])
                ndims.append(dim)
        res = DimPath(*ndims)
        if(not subtype is None):
            subtype = subtype._permuteDepDim(prevdims=res, permute_idxs=permute_idxs)
            return (res,subtype)
        else:
            return res#}}}

    def contigiousFixedNDims(self):#{{{
        """Returns number of contigious non-variable non-missing dims from 
        start of path"""
        depth = 0
        #maximize array size with fixed dims
        while(depth < len(self) and not self[depth].isVariable() and not 
                self[depth].has_missing):
            depth+=1
        return depth#}}}
    
    def matchDimPath(self, dimpath):#{{{#{{{
        """Matches dimpath exactly to dim paths in self.
        Returns a list containing positions of last dim
        """
        lastpos = []
        nseldims = len(dimpath.strip())
        if(dimpath and dimpath[0] is root):
            if(dimpath[-1] is end):
                if(self == dimpath[1:-1]):
                    lastpos.append(nseldims - 1)
            else:
                if(self[:(len(dimpath)-1)] == dimpath[1:]):
                    lastpos.append(nseldims - 1)
        elif(dimpath and dimpath[-1] is end):
            if(self[-(len(dimpath)-1):] == dimpath[:-1]):
                lastpos.append(len(self) - 1)
        elif(dimpath):
            pos = 0
            ndims = len(self)
            while(pos < ndims and (ndims - pos) >= nseldims):
                try:
                    curstart = self.index(dimpath[0], pos)
                except ValueError:
                    break
                if(self[(curstart + 1):(curstart + nseldims)] != dimpath[1:]):
                    pos = curstart + 1
                else:
                    lastpos.append(curstart + nseldims - 1)
                    pos = curstart + nseldims
        
        return lastpos#}}}#}}}

    def completePath(self):#{{{
        for pos in xrange(len(self)):
            if(len(self[pos]) > pos):
                return False
        return True#}}} 
                
    def splitSuffix(self,minlength=1):#{{{
        assert minlength <= len(self), "Minlength too large in splitSuffix"

        rev_self = self[::-1]
        rev_suffix = []
        curpos = 0
        while len(rev_suffix) < minlength:
            assert curpos < len(rev_self), "Cannot find complete suffix"
            dim = rev_self[curpos]
            minlength = max(len(dim) + curpos,minlength)
            rev_suffix.append(dim)
                
        return DimPath(*rev_suffix[::-1])                #}}}

def commonDimPath(dimpaths):#{{{
    """Returns common dimensions shared by all slices"""
    pos = 0
    minlen = min([len(dp) for dp in dimpaths])
    while(pos < minlen):
        cdim = set([dp[pos] for dp in dimpaths])
        if(len(cdim) != 1):
            break
        pos += 1
    return list(dimpaths)[0][:pos]#}}}

def uniqueDimPath(dimpaths,only_unique=True):#{{{
    """Returns unique dim path, i.e. at each nesting level determines
    if dim is unique and adds it to path. """

    if not dimpaths:
        return []
    fdims = set(dimpaths)

    if(len(fdims) == 1):
        path = fdims.pop()
    else:
        path = []
        for elems in itertools.izip_longest(*fdims):
            dimset = set([elem for elem in elems if not elem is None])
            if(len(dimset) == 1):
                path.append(dimset.pop())
            elif only_unique:
                return path
            else:
                path.append(dimset)
    return path#}}}


def orderAllDimPath(dimpaths):#{{{
    """Returns topologically ordered dim path, i.e. uses multiple dimpaths
    to maximally order dimensions. First uses order within slices, then
    order of slices."""

    if not dimpaths:
        return []
    fdims = set(dimpaths)
    if(len(fdims) == 1):
        path = fdims.pop()
        return path
    
    #step 1: create pairs for order within slice
    dimorder = toposort.StableTopoSortGraph()
    done = set()
    for dp in dimpaths:
        if dp in done:
            continue
        done.add(dp)
        dimorder.addNodesIfNotExist(*dp)
        for pos, d in enumerate(dp[1:]):
            if dp[pos] == d: #skip same dims after each other
                continue
            try:                
                dimorder.addEdge(dp[pos],d)
            except toposort.CycleError, e:
                pass
                
    seen_nodes = set(dimpaths[0])
    for dp in dimpaths[1:]:
        for dim in dp:
            if dim in seen_nodes:
                continue
            for ddim in seen_nodes:
                try:
                    dimorder.addEdge(ddim, dim)
                except toposort.CycleError, e:
                    pass
            seen_nodes.add(dim)    
                   
    #step 2: perform ordering
    path = []
    for level in dimorder.fullOrder():
        if len(level) == 1:
            path.extend(level)
        else:
            util.warning('Cannot order dims?')
            path.extend(ilevel)


    return path#}}}

def orderDimPath(dimpaths, keep_unordered=False):#{{{
    """Returns topologically ordered dim path, i.e. uses multiple dimpaths
    to maximally order dimensions. First uses order within slices, then
    order of slices."""

    if not dimpaths:
        return []
    fdims = set(dimpaths)
    if(len(fdims) == 1):
        path = fdims.pop()
        return path
    
    try:
        #step 1: create pairs for order within slice
        dimorder = toposort.StableTopoSortGraph()
        done = set()
        for dp in dimpaths:
            if dp in done:
                continue
            done.add(dp)
            dimorder.addNodesIfNotExist(*dp)
            for pos, d in enumerate(dp[1:]):
                if dp[pos] == d: #skip same dims after each other
                    continue
                dimorder.addEdge(dp[pos],d)
                    

        #step 3: perform ordering
        path = []
        pathdims = set()
        for level in dimorder.fullOrder():
            if len(level) == 1:
                path.extend(level)
                pathdims.update(level)
                continue
            try:
                #step 2: create pairs for order between slices
                sliceorder = toposort.StableTopoSortGraph()
                sliceorder.addNodesIfNotExist(*[d for d in dimpaths[0] if d in level])
                for dp in dimpaths[1:]:
                    for dim in dp:
                        if not dim in level:
                            continue
                        if sliceorder.hasNode(dim):
                            continue
                        dorder = list(sliceorder.nodes)                            
                        sliceorder.addNodeIfNotExist(dim)
                        for ddim in dorder:
                            sliceorder.addEdge(ddim, dim)

                for ilevel in sliceorder.fullOrder():
                    if len(ilevel) == 1:
                        path.extend(ilevel)
                        pathdims.update(ilevel)
                    else:
                        if keep_unordered:
                            path.append(ilevel)
                            pathdims.update(ilevel)
                        else:                    
                            util.warning('Cannot determine order of dimensions %s',str(ilevel))
                            return None
            except toposort.CycleError, e:
                if keep_unordered:
                    path.append([l for l in level if l not in path])
                    pathdims.update(path[-1])
                else:                    
                    util.warning('Cannot determine order of dimensions. %s',str(e))
                    return None
                                
    except toposort.CycleError, e:
        if keep_unordered:
            path.append([d for d in done_dims if d not in pathdims])
            pathdims.update(path[-1])
        else:
            util.warning('Cannot determine order of dimensions. %s',str(e))
            return None

    return path#}}}

def planBroadcastMatchPos(paths):#{{{
    """Matches dims in paths based on their position.
    Returns new set of dims and broadcast plan."""

    maxlen = max([len(path) for path in paths])
    plans = [[] for path in paths]
    bcdims = [None] * maxlen
    curpos = - 1
    while(-(curpos + 1) < maxlen):
        xdims = []
        l = set()
        for path in paths:
            if(len(path) > -(curpos+1)):
                xdims.append(path[curpos])
                l.add(path[curpos].shape)
            else:
                xdims.append(None)

        if(len(l) > 1):
            l.discard(1)
        if(len(l) > 1):
            l.discard(UNDEFINED)
        assert len(l) == 1, "Different shaped dimensions cannot be broadcast to each other"
        length = l.pop()
        nxdims = len(set(xdims))
        bcdim = [xdim for xdim in xdims if not xdim is None and xdim.shape == length][0]
        for xdim,plan in zip(xdims,plans):
            if(xdim is None):
                plan.append(BCNEW)
            elif(xdim.shape == 1 and bcdim.shape != 1):
                plan.append(BCEXIST)
            elif(xdim != bcdim):
                plan.append(BCENSURE)
            elif nxdims == 1:
                plan.append(BCCOPY)
            else:
                plan.append(BCSOURCE)
        bcdims[curpos] = bcdim 
        curpos -=1 
    for pos in xrange(len(plans)):
        plans[pos] = plans[pos][::-1]
    
    nbcdims = adaptDependentDims(bcdims, plans)

    return (nbcdims, bcdims, plans)        #}}}

def planBroadcastMatchDim(paths):#{{{
    """Matches dims in paths based on their identity, as well
    as their potential to be used as broadcast dimension (shape == 1).
    Returns new set of dims and broadcast plan"""
    graph = toposort.StableTopoSortGraph()

    #a dim can occur multiple times in dag, to prevent cycles
    #Therefore, it has to be represented by ids instead of itself
    #Step 1: assign ids to dims
    curid = 0
    translate = {}

    dimnames = dict()
    for dim in itertools.chain(*paths):
        if not dim in translate:
            translate[dim] = [curid]
            curid += 1
            if(dim.name in dimnames and dimnames[dim.name] != dim):
                raise RuntimeError, "Cannot broadcast, there are different dimensions with same name: " + str(dim.name) + ". Please rename (DimRename) or state their equivalence."
            dimnames[dim.name] = dim
    
    #broadcast dim mergers
    wildcard_links = dict()
    free_wildcards = set()
    
    #Step 2: create graph
    for path in paths:
        lastid = None
        for pathpos, dim in enumerate(path[::-1]):
            dimid = translate[dim][0]
            newnode = False

            #new wildcard dim?
            if(False and dim.shape == 1 and dim.has_missing is False and not dimid in wildcard_links):
                odimid = dimid
                dimids = graph.getParents(lastid)
                if(dimids):
                    dimid = min(dimids) #maps to first ordered dim (i.e firstmost path)
                    wildcard_links[odimid] = dimid
                else:
                    graph.addNodeIfNotExist(dimid)
                    free_wildcards.add(dimid)
            elif(dimid in wildcard_links): #old wildcarddim, use previous mapping
                dimid = wildcard_links[dimid]
            else: #normal dim, ensure it available in the graph
                newnode = graph.addNodeIfNotExist(dimid)

            if(pathpos > 0):
                pos = len(translate[dim])
                #try to add edge, such that we get no cycle
                #if cycle is found, add new ids for target node
                while True:
                    try:
                        graph.addEdge(dimid,lastid)
                        break
                    except toposort.CycleError, e:
                        pos -= 1
                        if(pos < 0):
                            translate[dim].append(curid)
                            newnode = graph.addNodeIfNotExist(curid)
                            curid += 1
                            pos = -1
                        dimid = translate[dim][pos]
            
            if(free_wildcards and newnode):
                for nodeid in list(graph)[::-1]:
                    if(nodeid in free_wildcards and not nodeid == dimid):
                        try:
                            graph.mergeNodes(nodeid,dimid)
                            wildcard_links[nodeid] = dimid
                            free_wildcards.discard(nodeid)
                            break
                        except toposort.CycleError, e:
                            pass #cannot be merged, let it be
                    
            lastid = dimid 
  
    #Step 3: get ordered bc dims
    rev_translate = dict(sum([[(subid,dim) for subid in id] for dim,id in translate.iteritems()],[]))
    bcdims = list(graph)
    bcdims = [rev_translate[bcdimid] for bcdimid in bcdims]

    #Step 4: create plans for each path
    plans = []
    for path in paths:
        plan = []
        pathpos = len(path) - 1
        for bcdim in bcdims[::-1]:
            if(pathpos >= 0):
                if( bcdim == path[pathpos]):
                    plan.append(BCCOPY)
                    pathpos -= 1
                else: 
                    for dimid in translate[path[pathpos]][::-1]:
                        if(dimid in wildcard_links):
                            bdim = rev_translate[wildcard_links[dimid]]
                            if(bdim == bcdim):
                                plan.append(BCEXIST)
                                pathpos -= 1
                                break
                    else:
                        plan.append(BCNEW)
            else:
                plan.append(BCNEW)

        plans.append(plan[::-1])
    
    #Step 5:discriminate between BCCOPY and BCSOURCE
    for planpos in xrange(len(plans[0])):
        x = set([plan[planpos] for plan in plans])
        if BCCOPY in x and len(x) > 1:
            for plan in plans:
                if plan[planpos] == BCCOPY:
                    plan[planpos] = BCSOURCE
    
    nbcdims = adaptDependentDims(bcdims, plans)
             
    return (nbcdims,bcdims, plans)#}}}

def adaptDependentDims(bcdims, plans):    
    nbcdims = list(bcdims)
    for pos, bcdim in enumerate(bcdims):
        if not bcdim.dependent:
            continue
        for plan in plans:
            if not plan[pos] == BCSOURCE:
                continue
            dep = bcdim.dependent
            ndep = applyPlan(list(dep), plan[max(0, pos - len(dep)):pos][::-1], newvalue=False, existvalue=False)
            while not ndep[-1]:
                ndep.pop()
            nbcdim = bcdim.changeDependent(tuple(ndep),nbcdims[max(0, pos - len(ndep)):pos][::-1])
            nbcdims[pos] = nbcdim
    return nbcdims

def planBroadcastFromPlan(path, plan, origdims, bcdims):
    plan = plan[::-1]
    bcdims = bcdims[::-1]
    ipath = path[::-1]
    origdims = origdims[::-1]

    existpos = [None] * len(plan)
    pathpos = 0
    origdimpos = 0
    
    #determine original dim positions
    for pos, (planelem, bcdim) in enumerate(zip(plan, bcdims)):
        if planelem == BCEXIST or planelem == BCENSURE:
            try: 
                pathpos = ipath.index(origdims[origdimpos], pathpos)
                existpos[pos] = pathpos
            except ValueError:
                pass
            origdimpos += 1
        elif planelem == BCCOPY or planelem == BCSOURCE:
            try:
                pathpos = ipath.index(bcdim, pathpos)
                existpos[pos] = pathpos
            except ValueError:
                pass
            origdimpos += 1
     
    #create new plan
    nplan = []
    nbcdims = []
    pathpos = 0
    for pos, (planelem, bcdim) in enumerate(zip(plan, bcdims)):
        nposses = [x for x in existpos[pos:] if not x is None]
        if nposses:
            nextpos = nposses[0]
        else:
            nextpos = len(ipath)
        if planelem == BCNEW:
            if pathpos < nextpos and bcdim in ipath[pathpos:nextpos]:
               xpos = ipath.index(bcdim, pathpos, nextpos)
               while pathpos <= xpos:
                    nplan.append(BCSOURCE)
                    nbcdims.append(ipath[pathpos])
                    pathpos += 1
            else:
                nplan.append(BCNEW)
                nbcdims.append(bcdim)
        elif planelem == BCEXIST or planelem == BCENSURE or planelem == BCCOPY or planelem == BCSOURCE:
            if existpos[pos] is None:
                nplan.append(BCNEW)
                nbcdims.append(bcdim)
            else:
                while pathpos < existpos[pos]:
                    nplan.append(BCSOURCE)
                    nbcdims.append(ipath[pathpos])
                    pathpos += 1
                nplan.append(planelem)
                nbcdims.append(bcdim)
                pathpos += 1
        else:
            raise RuntimeError, "Unknown path element"
       
    return (nbcdims[::-1],nplan[::-1])

def processPartial(bcdims, plans):
    nplans = list(plans)
    for i in range(len(bcdims)):
        pos = -(i+1)
        if bcdims[pos].dependent:
            break
        if(plans[pos] == BCNEW):
            nplans[pos] = BCCOPY
        elif(plans[pos] == BCEXIST):
            nplans[pos] = BCSOURCE
    return nplans

def applyPlan(seq,plan,newvalue=None,copyvalue=NOVAL,existvalue=NOVAL,ensurevalue=NOVAL,sourcevalue=NOVAL):#{{{
    elempos = 0
    nseq = []
    for planelem in plan:
        if(planelem == BCNEW):
            nseq.append(newvalue)
        elif(planelem == BCEXIST):
            if(existvalue is NOVAL):
                nseq.append(seq[elempos])
            else:
                nseq.append(existvalue)
            elempos += 1
        elif(planelem == BCCOPY):
            if(copyvalue is NOVAL):
                nseq.append(seq[elempos])
            else:
                nseq.append(copyvalue)
            elempos += 1
        elif(planelem == BCSOURCE):
            if(sourcevalue is NOVAL):
                nseq.append(seq[elempos])
            else:
                nseq.append(sourcevalue)
            elempos += 1
        elif(planelem == BCENSURE):
            if(copyvalue is NOVAL):
                nseq.append(seq[elempos])
            else:
                nseq.append(ensurevalue)
            elempos += 1
        else:
            raise RuntimeError, "Unknown plan type"
    nseq.extend(seq[elempos:])
    return nseq#}}}



def flatFirstDims(array,ndim):#{{{
    """Flattens first ndim dims in numpy array into next dim"""
    oshape = array.shape
    ndim = ndim + 1
    rshape = (int(numpy.multiply.reduce(array.shape[:ndim])),) + array.shape[ndim:]
    return array.reshape(rshape)#}}}

def createDimSet(sourcepaths):#{{{
    """Create set of all dims in sourcepaths"""
    dimset = set()
    for dimpath in sourcepaths:
        dimset.update(dimpath)
    return dimset#}}}

def createDimParentDict(sourcepaths):#{{{
    """Create dictionary containing for each dim in sourcepaths 
    the previous occuring dims  in a  list"""
    parents = defaultdict(list)
    for dimpath in sourcepaths:
        parents[dimpath[0]].append(None)
        for pos in xrange(1,len(dimpath)):
            parents[dimpath[pos]].append(dimpath[pos-1])
    return parents#}}}

def convertDoubleDim(paths):
    translate = {}
    npaths = []
    for path in paths:
        already_seen = set()
        npath = []
        for dim in path:
            while dim in already_seen:
                if not dim in translate:
                    translate[dim] = dim.copy(reid=True)
                dim = translate[dim]
            already_seen.add(dim)
            npath.append(dim)
        npaths.append(DimPath(*npath))
    return (npaths,translate)

def deconvertDoubleDim(paths, translate):
    translate = dict([(v,k) for k,v in translate.iteritems()])
    npaths = []
    for path in paths:
        npath = []
        for dim in path:
            while dim in translate:
                dim = translate[dim]
            npath.append(dim)
        npaths.append(DimPath(*npath))
    return npaths

def selectOrderDim(paths, idx):
    paths,translate = convertDoubleDim(paths)
    opath = orderAllDimPath(paths)
    try: 
        dim = opath[idx]
    except IndexError:
        raise RuntimeError, "Dimension at depth " + str(dim_selector) + " does not exist"

    selpath = extendParentDim(DimPath(dim), [DimPath(root, *p) for p in paths], -1)
    selpaths = deconvertDoubleDim([selpath],translate)
    return selpaths[0]

def getOrderDim(paths):
    paths,translate = convertDoubleDim(paths)
    opath = orderAllDimPath(paths)
    selpaths = deconvertDoubleDim([opath],translate)
    return selpaths[0]

def extendParentDim(path, sourcepaths, length=1):#{{{
    if length >= 0:
        length -= len(path.strip())   
        if(length <= 0):
            return path
    if path[0] is root:
        raise RuntimeError, "Could not get long enough dim path"

    ndims = []
    xpaths = set()
    for spath in sourcepaths:
        lastposs = spath.matchDimPath(path)
        for lastpos in lastposs:
            r = spath[:(lastpos + 1 - len(path))]
            xpaths.add(r)
        
    while(all([len(xpath) > 0 for xpath in xpaths])and (len(ndims) < length or length == -1)):
        xdims = set()
        nxpaths = set()
        for xpath in xpaths:
           xdims.add(xpath[-1])
           nxpaths.add(xpath[:-1])
        
        if(not xdims or len(xdims) > 1):
            if length == -1:
                break
            else:
               raise RuntimeError, "Cannot find unique parent for dim: " + str(path[0])
        xpaths = nxpaths
        ndims.append(xdims.pop())

    path = DimPath(*(ndims[::-1])) + path
    return path
    #}}}

def getArrayDimPathFromType(rtype):
    if(rtype.__class__ is rtypes.TypeArray):
        return rtype.dims + getArrayDimPathFromType(rtype.getSubType())
    else:
        return DimPath()

def getNestedArraySubType(rtype,depth):
    if depth == 0:
        return rtype
    if(rtype.__class__ is rtypes.TypeArray):
        depth -= len(rtype.dims)
        if depth < 0:
            return rtype
        else:
            return getNestedArraySubType(rtype.getSubType(),depth)
    else:
        return rtype

def dimsToArrays(dims, subtype):
    dims = dims[::-1]
    subtype = rtypes.TypeArray(dims=dims[:1],subtypes=(subtype,))

    for pos in xrange(1,len(dims)):
        subtype = rtypes.TypeArray(dims=dims[pos:(pos+1)],subtypes=(subtype,))
    return subtype

class DimPathRoot(Dim):
    pass
root = DimPathRoot(UNDEFINED, name="_root_")

class DimPathEnd(Dim):
    pass
end = DimPathEnd(UNDEFINED, name="_end_")

def identifyUniqueDimPathSource(source,dim_selector):#{{{
    res = identifyDimPathSource(source,dim_selector)
    if(len(res) > 1):
        raise RuntimeError, "Cannot choose between dim paths: " + str(res)
    elif(len(res) == 0):
        raise RuntimeError, "No matching dimpath found for selector: " + str(dim_selector)
    return res.pop()#}}}

def identifyDimPathSource(source,dim_selector):#{{{
    if(isinstance(dim_selector, context.Context)):
        dim_selector = context._apply(dim_selector, source)
    elif(isinstance(dim_selector, basestring)):
        if(dim_selector in [s.name for s in source._slices] or dim_selector[0].isupper()):
            dim_selector = getattr(source,dim_selector)

    if(isinstance(dim_selector, representor.Representor)):
        return set([s.dims for s in dim_selector._slices])

    return identifyDimPath(list([s.dims for s in source._slices]),dim_selector)#}}}

def identifyUniqueDimPath(source,dim_selector):#{{{
    res = identifyDimPath(source,dim_selector)
    if(len(res) > 1):
        raise RuntimeError, "Cannot choose between dim paths: " + str(res)
    elif(len(res) == 0):
        raise RuntimeError, "No matching dimpath found for selector: " + str(dim_selector)
    return res.pop()#}}}

def identifyDimPath(sourcepaths, dim_selector):#{{{
    if(isinstance(dim_selector, int)):
        return set([selectOrderDim(sourcepaths, dim_selector)])
    elif(isinstance(dim_selector, long)):
        d = commonDimPath(sourcepaths)

        try:
            d[dim_selector]
        except IndexError:
            raise RuntimeError, 'Not enough  common dimensions shared by slices, but attempting to select such a dimension'

        path = DimPath(* ((root,) + d[:dim_selector] + (d[dim_selector],)))
        return set([path])

    if(isinstance(dim_selector, Dim)):
        #fixme: root/end
        return set([DimPath(dim_selector)])
    elif(dim_selector is None):
        v = selectOrderDim(sourcepaths, -1)
        return set([selectOrderDim(sourcepaths, -1)])
        #return set([commonDimPath(sourcepaths)])
    elif(isinstance(dim_selector, basestring)):
        return identifyDimPathParse(sourcepaths, dim_selector)
    elif(isinstance(dim_selector, DimPath)):
        #fixme: root/end
        return set([dim_selector])
    elif(isinstance(dim_selector, set)):
        res = set()
        for elem in dim_selector:
            res.update(identifyDimPath(sourcepaths, elem))
        return res 
    else:
        raise RuntimeError, "Unexpected dim selector: " + str(dim_selector)
    return res#}}}

def identifyDimPathParse(sourcepaths, dim_selector):#{{{
    dim_sel = []
    if(dim_selector[0] == "^"):
        dim_selector = dim_selector[1:]
        dim_sel.append(root)

    if(dim_selector and dim_selector[-1] == "$"):
        dim_selector = dim_selector[:-1]
        fixed_end = True
    else:
        fixed_end=False

    if(dim_selector):
        dim_sel.extend(dim_selector.split(":"))
    
    if(fixed_end):
        dim_sel.append(end)

    res = set()
    for spath in sourcepaths:
        res.update(identifyDimPathParseHelper(spath, dim_sel))
    return res#}}}

def identifyDimPathParseHelper(spath, dim_sel, outer=True):#{{{
    res = set()
    ds = dim_sel[0]
    
    if(ds is root):
        assert outer is True, "Root dim selection after begin"
        if(not dim_sel[1:]):
            res.add((spath[0],))
        else:
            _processRest(spath, dim_sel[1:], res, (root,))
    elif(ds is end):
        assert len(dim_sel) == 1, "Dim selectors found after end selector"
        res.add(tuple(spath) + (end,))
    elif(ds == "?"):
        if outer:
            for pos in xrange(len(spath)-1):
                _processRest(spath[(pos+1):], dim_sel[1:], res, (spath[pos],))
        elif(spath):
            _processRest(spath[1:], dim_sel[1:], res, (spath[0],))
    elif(ds == "*"):
        for pos in xrange(len(spath)):
            _processRest(spath[pos:], dim_sel[1:], res, spath[:pos])

    elif(ds == "+"):
        for pos in xrange(1,len(spath)):
            _processRest(spath[pos:], dim_sel[1:], res, spath[:pos])
    else:
        if outer:
            for pos in xrange(len(spath)):
                if(spath[pos].name == ds):
                    _processRest(spath[(pos+1):], dim_sel[1:], res, (spath[pos],))
        else:    
            if(spath and spath[0].name == ds):
                _processRest(spath[1:], dim_sel[1:], res, (spath[0],))
        
    return res        #}}}

def _processRest(spath, dim_selector, res, pathprefix):#{{{
    if(not dim_selector):
        if(pathprefix):
            res.add(DimPath(*pathprefix))
        return

    pathsuffixes = identifyDimPathParseHelper(spath, dim_selector,outer=False)
    for s in pathsuffixes:
        res.add(DimPath(*(pathprefix + s)))
        return#}}}


#}}}



import operator
import numpy

from ..constants import *
from ..utils.multi_visitor import VisitorFactory, NF_ERROR, NF_ELSE
from ..utils.missing import Missing

_delay_import_(globals(),"..utils","cutils","util","sparse_arrays")

class slicetuple(tuple):
    def __new__(self,s):
        res = tuple.__new__(self,(s.start,s.stop,s.step))
        return res

    def getslice(self):
        return slice(*self)


class RTypeFreezeProtocol(VisitorFactory(prefixes=("needFreeze", "freeze","execFreeze","ftype","needUnfreeze","unfreeze","execUnfreeze","uftype"), 
                                                        flags=NF_ELSE)):

    #Determine if necessary to freeze a type (or its subtypes)    
    def need_freeze(self, rtype):
        return True
    needFreezeTypeAny=need_freeze
    needFreezeTypeDict=need_freeze
    needFreezeTypeArray=need_freeze
    needFreezeTypeSlice=need_freeze
    needFreezeTypeSet=need_freeze

    def noneed_freeze(self, rtype):
        return False
    needFreezeTypeString=noneed_freeze
    needFreezeTypeScalar=noneed_freeze
    
    def needFreezeTypeTuple(self,rtype):
        return any([self.needFreeze(subtype)for subtype in rtype.subtypes])
    
    #freeze standard python objects
    def ftypeobject(self, obj):
        if(hasattr(obj, "__freeze__")):
            return obj.__freeze__()
        else:
            return obj

    def _ftypenofreeze(self,obj):
        return obj
    ftypeint=_ftypenofreeze
    ftypefloat=_ftypenofreeze
    ftypestr=_ftypenofreeze
    ftypeunicode=_ftypenofreeze

    def ftypetuple(self, obj):
        return tuple([self.ftype(elem) for elem in obj])

    def _ftypesequence(self, obj):
        return util.darray([self.ftype(elem) 
                for elem in obj]).view(util.farray) 
    ftypelist = _ftypesequence

    def ftypendarray(self, obj):
        if(obj.dtype == object):
            return util.darray([self.ftype(elem) 
                for elem in obj]).view(util.farray) 
        else:
            return obj.view(util.farray)

    def ftypeset(self, obj):
        return frozenset(obj)

    def ftypeslice(self, obj):
        return slicetuple(obj)

    def map(self, func, parent_type, seq):
        if(parent_type.has_missing):
            seqres = []
            for elem in seq:
                if(elem is Missing):
                    seqres.append(Missing)
                else: 
                    seqres.append(func(elem))
        else:
            seqres = [func(elem) for elem in seq]
        
        seqres = util.darray(seqres,seq.dtype)
        return seqres 
    

    def execFreezeTypeUnknown(self, ptype, seq):
        seq = self.map(self.ftype, ptype, seq)
        return seq
   
    def execFreezeTypeScalar(self, ptype, seq):
        return seq


    def execFreezeTypeSlice(self, ptype, seq):
        return self.map(self.ftypeslice, ptype, seq)
        

    def execFreezeTypeTuple(self, ptype, seq):
        if(not ptype.subtypes):
            return self.execFreezeTypeUnknown(ptype, seq)
        else:
            return self.map(self.ftypetuple, ptype, seq)
        
        columns = []
        if(len(ptype.subtypes) > ptype.min_len):
            l = self.map(len, otype=int, out_empty = 0, has_missing=self.detector.hasMissing())

        for pos, subtype in enumerate(rtype.subtypes):
            f = operator.itemgetter(pos)
            if(pos < rtype.min_len):
                subseq = seq.map(f,otype=object,out_empty=Missing,has_missing=rtype.has_missing)
            else:
                subseq = seq.sparse_filter(l > pos).map(f,out_empty=Missing,otype=object,has_missing=True)
            columns.append(self.execFreeze(subtype, subseq))

        nseq = util.darray(zip(*columns))
        nseq.shape = seq.shape
        seq = sparse_arrays.FullSparse(nseq)
        return seq
    
    #def execFreezeTypeDict(self,rtype,seq):
    #    if(self.needFreeze(rtype)):
    #        if(not rtype.subtypes):
    #            return self.execFreezeTypeUnknown(rtype,seq)
    #        
    #        columns = []
    #        if(len(rtype.subtypes) > rtype.min_len):
    #            l = seq.map(len, otype=int, out_empty = 0, has_missing=self.detector.hasMissing())

    #        for pos, (fieldname, subtype) in enumerate(zip(rtype.fieldnames, rtype.subtypes)):
    #            def getname(elem):
    #                try:
    #                    return elem[name]
    #                except (KeyError, TypeError):
    #                    return Missing
    #            subseq = seq.map(getname,otype=object,out_empty=Missing,has_missing=rtype.has_missing)
    #            columns.append(self.execFreeze(subtype, subseq))

    #        nseq = util.darray(zip(*columns))
    #        nseq.shape = seq.shape
    #        seq = sparse_arrays.FullSparse(nseq)
    #    return seq
    #
    #def execFreezeTypeNamedTuple(self,rtype,seq):
    #    if(self.needFreeze(rtype)):
    #        if(not rtype.subtypes):
    #            return self.execFreezeTypeUnknown(rtype,seq)
    #        
    #        columns = []
    #        if(len(rtype.subtypes) > rtype.min_len):
    #            l = seq.map(len, otype=int, out_empty = 0, has_missing=self.detector.hasMissing())

    #        for pos, (fieldname, subtype) in enumerate(zip(rtype.fieldnames, rtype.subtypes)):
    #            def getname(elem):
    #                try:
    #                    return elem.name
    #                except (KeyError, TypeError):
    #                    return Missing
    #            subseq = seq.map(getname,otype=object,out_empty=Missing,has_missing=rtype.has_missing)
    #            columns.append(self.execFreeze(subtype, subseq))

    #        nseq = util.darray(zip(*columns))
    #        nseq.shape = seq.shape
    #        seq = sparse_arrays.FullSparse(nseq)
    #    return seq

    def execFreezeTypeArray(self, ptype, seq):
        subtype = ptype.subtypes[0]
        if(self.needFreeze(subtype)):
            def subfreeze(elem):
                elem = self.execFreeze(subtype, elem)
                elem = elem.view(util.farray)
                return elem
        else:
            def subfreeze(elem):
                try:
                    elem = elem.view(util.farray)
                except AttributeError, e:
                    elem = numpy.array(elem).view(util.farray)
                return elem
        seq = self.map(subfreeze, ptype, seq)
        return seq
     
    def execFreezeTypeString(self,ptype,seq):
        return seq
    
    def execFreezeTypeSet(self, ptype, seq):
        seq = self.map(frozenset, ptype, seq)
        return seq
    
freeze_protocol = RTypeFreezeProtocol()




from ibidas.utils import util
_delay_import_(globals(),"..ops")
_delay_import_(globals(),"..query_graph")

class Expression(object):
    __slots__ = ["etype","eobj","in_slices","out_slices","all_slices"]
    def __init__(self, etype, eobj):
        self.etype = etype
        self.eobj = eobj
        self.in_slices = set()
        self.out_slices = set()
        self.all_slices = set()

    def addInSlice(self, in_slice):
        self.in_slices.add(in_slice)

    def addOutSlice(self, out_slice):
        self.out_slices.add(out_slice)

    def addAllSlice(self, between_slice):
        self.all_slices.add(between_slice)

    def sourceDataEdge(self, edge, graph):
        links = graph.node_attributes['links']
        while(links[edge.source] == self):
            edge = graph.getDataEdge(edge.source)
        return edge
    
    def targetDataNode(self, node, graph, opfilter = set()):
        links = graph.node_attributes['links']
       
        nsource = node
        while True:
            edges = [edge for edge in graph.edge_source[nsource] if isinstance(edge,query_graph.ParamEdge)]
            edge = edges[0]
            if len(edges) > 1:
                assert all([links[edge.target] != self for edge in edges]), "Cannot decide on target data node"
            nsource = edge.target

            if opfilter and not nsource.__class__ in opfilter:
                break
            
            if links[nsource] != self:
                break

        return edge.source
   
    def getNodesByClass(self, cls):
        return [node for node in self.all_slices if node.__class__ == cls]

    def getNodeByClass(self, cls):
        res = self.getNodesByClass(cls)
        assert len(res) ==1, "No nodes or multiple nodes with the same class found"
        return res[0]

    def __str__(self):
        return str(self.etype)

class BinFuncElemExpression(Expression):
    def getOp(self):
        return (self.getOutSlice().sig.name, self.eobj.__class__.__name__)

    def getOutSlice(self):
        return self.getNodeByClass(ops.BinFuncElemOp)


    def getLeftRightInEdges(self, graph):
        c = self.getNodeByClass(ops.BinFuncElemOp)
        e1 = self.sourceDataEdge(graph.getDataEdge(c,0), graph)
        e2 = self.sourceDataEdge(graph.getDataEdge(c,1), graph)
        return (e1,e2)

    def getOutEdges(self, graph):
        c = self.getNodeByClass(ops.BinFuncElemOp)
        out_edges = graph.edge_source[c]
        return out_edges

class FilterExpression(Expression):
    def getDims(self):
        filters = self.getNodesByClass(ops.FilterOp)
        return set([filter.dims for filter in filters])
        
    
    def getInfo(self, graph):
        filters = self.getNodesByClass(ops.FilterOp)
        cedge = self.sourceDataEdge(graph.getDataEdge(filters[0],name="constraint"), graph)
        fedges = [self.sourceDataEdge(graph.getDataEdge(f), graph) for f in filters]
        onodes = [self.targetDataNode(f, graph) for f in filters]
        return (fedges,cedge, onodes)


class MatchExpression(Expression):
    def getUsedSources(self):
        sops = self.getNodesByClass(ops.SelectOp)
        res = [False,False]
        for sop in sops:
            res[sop.index] = True
        return res
       
    def getSop(self, index):
        sops = self.getNodesByClass(ops.SelectOp)
        for sop in sops:
            if sop.index == index:
                return sop
        else:
            raise RuntimeError, "Could not find sop"
    
    def getDim(self,index, graph):
        sop = self.getSop(index)
        filters = [edge.target for edge in graph.edge_source[sop]]
        assert all([isinstance(filter,ops.FilterOp) for filter in filters]), "Non-filter target of select in match"
        dims = set([filter.dims for filter in filters])
        return dims

    def getType(self):
        sops = self.getNodeByClass(ops.EquiJoinIndexOp)
        return sops.jointype
    
    def getComparisonEdges(self,graph):
        jindex = self.getNodeByClass(ops.EquiJoinIndexOp)
        ledge = self.sourceDataEdge(graph.getDataEdge(jindex,pos=0),graph)
        redge = self.sourceDataEdge(graph.getDataEdge(jindex,pos=1),graph)
        return (ledge,redge)


    def getInfo(self, index, graph):
        sop = self.getSop(index)
        filters = [edge.target for edge in graph.edge_source[sop]]
        edges = [self.sourceDataEdge(graph.getDataEdge(filter,0),graph) for filter in filters]
        onodes = [self.targetDataNode(filter,graph,set([ops.FilterOp, ops.UnpackArrayOp])) for filter in filters]

        return (edges,onodes)

       
